/* Copyright (C) 2018- SofTek Systems, Inc. */

//ajaxのタイムアウト時間。脆弱性別状況・脆弱性詳細・ホスト別状況・ホスト詳細画面で使用する。
const AJAX_TIMEOUT_SECOND = 30000;
const DB_MAX_INT_NUM = 2147483647; //DBのIntergerの最大数値
const DB_MIN_INT_NUM = -2147483648 //DBのIntergerの最小数値

//ブラウザ情報
var browser_ident = '';
var user_agent = window.navigator.userAgent.toLowerCase();
if(user_agent.indexOf('msie') != -1 || user_agent.indexOf('trident') != -1) {
    //Internet Explorer
} else if(user_agent.indexOf('edge') != -1) {
    //Edge;
} else if(user_agent.indexOf('chrome') != -1) {
    browser_ident = 'Chrome';
} else if(user_agent.indexOf('safari') != -1) {
    browser_ident = 'Safari';
} else if(user_agent.indexOf('firefox') != -1) {
    browser_ident = 'FireFox';
}
const Browser_Ident = browser_ident;

// 配列同士の比較
Array.prototype.compare = function(e) {
    if (!(e instanceof Array)) {
	throw new TypeError('argument type is not array.');
    }
    var a = JSON.stringify(this.sort());
    var b = JSON.stringify(this.sort());
    return (a === b);
};

//詳細以外のページに遷移した場合は、操作した行に色をつけないようにするための処理
window.addEventListener('DOMContentLoaded', function(){
    const hostDetailClickedRowCookie = 'clicked_hostid';
    const vulDetailClickedRowCookie = 'clicked_vul_id'; 
    const current_href = location.href;
    const colorRowNecessaryPages = new RegExp('/sidfm/host/|/sidfm/vulnerability/');
    if( !current_href.match(colorRowNecessaryPages) ){
        //removeCookieで消えない時は{ expires: -1, path: '/' }で消す。
        //{ expires: -1, path: '/' }で消えないときはremoveCookieで消す。
        $.cookie(hostDetailClickedRowCookie, '', { expires: -1, path: '/' });
        $.removeCookie(hostDetailClickedRowCookie);
        $.cookie(vulDetailClickedRowCookie, '', { expires: -1, path: '/' });
        $.removeCookie(vulDetailClickedRowCookie);
    }
});

// 時系列の切り替えを有効化
// #pages_form内に、#pages_timeline_risk_period, #pages_timeline_status_period 及び controllerの処理 が必須
function enable_toggle_timeline_switch(){
    const toggle = (selector, value) => {
        $(selector).val(value);
        $('#pages_form').submit();
    };

    $('[data-area="timeline"][data-timeline="risk"] [data-period]').on('click', function(){
        toggle('#pages_form [name="pages[timeline_risk_period]"]', $(this).attr('data-period'));
    });


    $('[data-area="timeline"][data-timeline="status"] [data-period]').on('click', function(){
        toggle('#pages_form [name="pages[timeline_status_period]"]', $(this).attr('data-period'));
    });
}

//サマリー表の表示件数の設定
//SidfmControllerSupport#get_summary_table_linesで値を取得する
function set_summary_table_lines(page_name, num){
    var target = 'sidfm_' + page_name + '_summary_num';
    $.cookie(target, num);
}

function get_summary_table_lines(page_name){
    var target = 'sidfm_' + page_name + '_summary_num';
    return $.cookie(target);
}

//カテゴリ列の表示順の切り替え
function toggleSummaryPriorityCategory(){
    const cookieName = 'sidfm_summary_prior_category';
    const system = 'system';
    const custom = 'custom';
    if( !$.cookie(cookieName) ){
        $.cookie(cookieName, system);
    }

    const current = $.cookie(cookieName);
    const toggleButton = $('th .toggle-summary-priority-category');
    toggleButton.on('click', function(e){
        e.stopPropagation();
        if( current === system ){
            $.cookie(cookieName, custom);
        }else{
            $.cookie(cookieName, system);
        }
        const form = $('#pages_form');
        if( form.length > 0 ){
            form.submit();
        }else{
            location.reload(true);
        }
    });
}

// svg のエレメントを返す。
// 必ずこれを使用すること。jqueryでエレメントを生成すると、Google Chrome以外で出力されないため。
function $s(elem) {
    return $(document.createElementNS('http://www.w3.org/2000/svg', elem));
}

// 初期化
function init_sidfm() {
    logging_error();
    enable_bootstrap_actions(null);
    force_click_ignore();
    header_title_update();
    //bootstrapのpopoverを、modalが開くときに閉じる。
    $('.modal').on('show.bs.modal', function(){
        $('[data-toggle="popover"]').popover('hide');
    });
   
    //resizeで消去
    $(window).on('resize',function(){
        $('[data-toggle="dropdown"]').dropdown('hide');
    });
}

// 指定したtarget配下についてtooltip, popoverを有効にする。
// targetに!targetがtrueになる値(null等)を設定すると、全体が対象となる。
function enable_bootstrap_actions(target) {
    if (!target) { target = ''; } else { target = target + ' '; }
    $(target + '[data-toggle="datepicker"]').datepicker({
	currentText: '今日',
	monthNames: [ '1月', '2月', '3月', '4月', '5月', '6月', '7月', '8月', '9月', '10月', '11月', '12月' ],
	monthNamesShort: [ '1月', '2月', '3月', '4月', '5月', '6月', '7月', '8月', '9月', '10月', '11月', '12月' ],
	dayNames: [ '日曜日', '月曜日', '火曜日', '水曜日', '木曜日', '金曜日', '土曜日' ],
	dayNamesShort: [ '日', '月', '火', '水', '木', '金', '土' ],
	dayNamesMin: [ '日', '月', '火', '水', '木', '金', '土' ],
	weekHeader: '週',
	dateFormat: 'yy/mm/dd',
	firstDay: 0,
	showButtonPanel: true,
	showMonthAfterYear: true,
	yearSuffix: '年',
	closeText: '閉じる'
    });
    // datepickerの「閉じる」ボタンでメニューまで閉じないようにする。
    $('#ui-datepicker-div').on('click', function(e) {
	e.stopPropagation();
	return false;
    });
    $(target + ' [data-toggle="dropdown"]').dropdown();
    $('[data-toggle="dropdown"]').parent().on('hide.bs.dropdown', function(){
        $('[data-toggle="datepicker"]').datepicker('hide'); 
    });
    $(target + '[data-toggle="tooltip"]').tooltip();
    $(target + '[data-toggle="popover"]').popover();
}

// paginator用アクション
function init_paginator(base = 'pages') {
    $('.' + base + '_change_offset_paging').on('click', function() {
	$('#' + base + '_offset').val($(this).data('offset'));
	$('#' + base + '_form').submit();
	return false;
    });
    $('.' + base + '_change_per_page').on('change', function() {
	$('#' + base + '_per_page').val($(this).val());
	$('#' + base + '_form').submit();
	return false;
    });
    $('.' + base + '_change_offset_direct').on('keypress', function(e) {
	if (e.which == 13) {
	    var offset = parseInt($(this).val()) - 1;
	    //if (offset < 0) { offset = 1; }
	    if(offset < 0) { offset = 0; }
	    $('#' + base + '_offset').val(String(offset));
	    $('#' + base + '_form').submit();
	    return false;
	}
	return true;
  
    });
}

// sidebarについて、指定したidのアイテムをactiveにする
function activate_sidebar_item(id) {
  $('.nav-sidebar li').removeClass('active');
  $('#' + id).addClass('active');
}

// 選択された項目を別の項目に移動する。
// 2ペインのマルチセレクト入力用
//   p_selected_ids         : 選択された項目IDの一覧を保存している場所。hidden fieldのvalue等を想定。
//   p_current_selected     : 選択された項目のあるselectの場所。
//   p_opposite_selected    : 反対側の項目のあるselectの場所。
//   flg_reverse            : falseなら選択されたidを追加、trueなら選択されたidを除外。
function move_selected_items(p_selected_ids, p_current_selected, p_opposite_selected, flg_reverse) {
    var selected_ids = JSON.parse($(p_selected_ids).val());
    var options = $(p_current_selected + ' > option:selected');
    for (var i = 0; i < options.length; i++) {
        var option = $(options[i]);
        var s_id = parseInt(option.val());
        var s_text = option.text();
        var desc = option.data('title');
        if (flg_reverse) {
            selected_ids = selected_ids.filter(function(x, i, self) {
                return x != s_id;
            });
            selected_ids = selected_ids.filter(function(x, i, self) {
                return self.indexOf(x) == i;
            });
        } else {
            selected_ids.push(s_id);
            selected_ids = selected_ids.filter(function(x, i, self) {
                return self.indexOf(x) == i;
            });
        }
        $(p_selected_ids).val(JSON.stringify(selected_ids));
        $(p_opposite_selected).append(
            $('<option></option>', { value: s_id, text: s_text, class: 'text-truncate g-max-width-21em', 'data-toggle': 'tooltip', 'data-placement': 'right', 'data-title': desc, 'data-boundary': 'window' })
        );
    }
    $(p_current_selected + ' > option:selected').remove();
   
    return false;
}

// heatmapの出力。
// width, height はpx単位。
// width=360, height=180で10000件程度なら問題なく表示できることを確認。
// targetはid等の文字列(内部で$()で解釈します)、dataはjqueryのエレメント
// になることに注意。
function heatmap(target, width, height, data, callback) {
    if (width <= 0 || height <= 0) return;

    var rect_x = Math.floor(width * (3.0/100.0));
    var rect_y = Math.floor(width * (3.0/100.0));
    var rect_width = width - (rect_x * 2);
    var rect_height = height - (rect_y);

    var container = $('<div></div>');
    var svg = $s('svg').attr({
        xmlns: 'http://www.w3.org/2000/svg',
        'xmlns:xlink': 'http://www.w3.org/1999/xlink',
        preserveAspectRatio: 'xMinYMin meet',
        viewBox: '0 0 ' + width + ' ' + height,
        x: '0px',
        y: '0px',
        width: '' + width + 'px',
        height: '' + height + 'px',
        version: '1.1',
        'id': 'heatmap-svg'
    });
    container.append(svg);
    svg.append(
        $s('rect').attr({
            x: '' + rect_x + 'px',
            y: '' + rect_y + 'px',
            fill: '#f8f8f8',
            width: '' + rect_width + 'px',
            height: '' + rect_height + 'px',
        })
    );
  /*  var x_grid_count = Math.floor(Math.sqrt(data.length * (rect_width / rect_height)));
    var y_grid_count = Math.floor(data.length / x_grid_count) + 1;
    var x_grid_size = rect_width / x_grid_count;
    if (x_grid_size > ((0.0 + rect_width) * 30.0 / 100.0)) { x_grid_size = Math.floor((0.0 + rect_width) * 30.0 / 100.0); }
    var y_grid_size = rect_height / y_grid_count;
    if (y_grid_size > ((0.0 + rect_height) * 60.0 / 100.0)) { y_grid_size = Math.fllor((0.0 + rect_height) * 60.0 / 100.0); } */
    let data_length = data.length//$('#heatmap-svg a rect').length + data.length;
    if(rect_width >= rect_height){
        var x_grid_count = Math.ceil(Math.sqrt(data_length) * (rect_width / rect_height));
        var y_grid_count = Math.floor(data_length / x_grid_count) + 1;
        var x_grid_size = rect_width / x_grid_count;
        var y_grid_size = x_grid_size;
    }else{
        var x_grid_count = Math.ceil(Math.sqrt(data_length) * (rect_width / rect_height));
        var y_grid_count = Math.floor(data_length / x_grid_count) + 1;
        var x_grid_size = rect_height / y_grid_count;
        x_grid_count = Math.floor(rect_width / x_grid_size);
        var y_grid_size = x_grid_size;
    }

    setTimeout(function(){
        $(target).find('[data-toggle="tooltip"]').tooltip('dispose');
        $(target).html(container.html());
        let boxes = [];
        let i = 0;
        let count = 0;
        const add = 1000;
        const svg = $('#heatmap-svg');
        let is_appending = false;
        //for (var i = 0, len = data.length; i < len; i++) {
        const gradually_add_boxes = () => {
            if( count < data_length ){
                data.slice(count, count + add).forEach(item => {
                    var x_grid_position = i % x_grid_count;
                    var y_grid_position = Math.floor(i / x_grid_count);

                    var x_grid_left = rect_x + x_grid_size * x_grid_position;
                    var y_grid_top = rect_y + y_grid_size * y_grid_position;
                    var callback_result = callback.call(this, item);
                    var link = $s('a').attr({
                        /*'xlink:*/'href': callback_result.link,
                        'data-toggle': 'tooltip',
                        'data-container': 'body',
                        'data-placement': 'top',
                        'data-trigger': 'hover',
                        'data-html': 'true',
                        'title': callback_result.tooltip
                    });

                    var rect_opts = {
                        x: '' + x_grid_left + 'px',
                        y: '' + y_grid_top + 'px',
                        fill: callback_result.color,
                        width: '' + x_grid_size + 'px',
                        height: '' + y_grid_size + 'px',
                    };

                    // データが少ない場合のみ枠線あり。
                    if (data_length < 2000) {
                        rect_opts['stroke'] = 'black';
                        rect_opts['stroke-width'] = '0.5px';
                    }

                    link.append($s('rect').attr(rect_opts));
                    i += 1;
                    is_appending = true;
                    setTimeout(function(){
                        svg.append(link);
                        is_appending = false;
                    });
                });
                count += add;
                gradually_add_boxes();
            }else{
                let timer = setInterval(function(){
                    if( !is_appending ){
                        enable_bootstrap_actions(target);
                        clearInterval(timer);
                    }
                });
            }
        };
        gradually_add_boxes();
        // このようにしないとsvgが解釈されない。
        //$(target).html(container.html());
    });
    //enable_bootstrap_actions(target);
}

// グラディエーションの計算
// s, eはそれぞれ変換の始端・終端の色をHashで指定する。
//   { r: 1, g: 2, b: 3 }
// pは変化の割合(%)
function calc_gradient_color(s, e, p) {
    var ret = { r:0, g:0, b:0 };
    var keys = [ 'r', 'g', 'b' ];
    for (var i = 0; i < keys.length; i++) {
      ret[keys[i]] = Math.floor((e[keys[i]] - s[keys[i]]) * p / 100.0 + s[keys[i]]);
    }

    return ret;
}

// hex形式の色をcalc_gradient_colorでで使用する形式に変換
function from_hex_to_rgb(hex) {
    var res = ('' + hex).match(/^#([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/);
    var r = parseInt(res[1], 16);
    var g = parseInt(res[2], 16);
    var b = parseInt(res[3], 16);
    return { r:r, g:g, b:b };
}

// calc_gradient_colorで返すrgbのHashを色指定用の文字列(rgb)に変換
function to_rgb_str(s) {
    return 'rgb(' + s.r + ',' + s.g + ',' + s.b + ')';
}

// calc_gradient_colorで返すrgbのHashを色指定用の文字列(hex)に変換
function to_rgb_hex(s) {
    return '#' +
	[ 'r', 'g', 'b' ].map(function(i) {
	    return ('0' + s[i].toString(16)).slice(-2);
	}).join('');
}

// color_引数のidが存在する場合に、その値を取得して返す。
function symbol_color(color) {
    var c = $('#color_' + color);
    return c.length ? c.val() : c;
}

// SidfmHelper#status_barとほぼ同じ動作。
// modal のように動的処理で生成する必要のある場所で採用。   
function status_bar(width, height, data, unit, opts) {
    opts = (typeof opts === 'undefined') ? {} : opts;
    opts['width'] = '' + width + 'px';
    opts['height'] = '' + height + 'px';
    opts['xmlns'] = 'http://www.w3.org/2000/svg';
    opts['version'] = '1.1';
    var svg = $s('svg').attr(opts);
    var data_size = data.length;
    var value_sum = 0;
    for (var i = 0; i < data_size; i++) {
	value_sum += data[i]['value'];
    }
    var unit_width = parseFloat(width) / parseFloat(value_sum);
    var current_x = 0;
    for (var i = 0; i < data_size; i++) {
	var w = Math.floor(unit_width * parseFloat(data[i]['value']));
	var html = "<svg xmlns='http://www.w3.org/2000/svg' version='1.1' width='12px' height='12px'><rect x='0' y='0' width='10' height='12' fill='" + symbol_color(data[i]['color']) + "' /></svg>" + data[i]['name'] + ":" + data[i]['value'] + unit;
	var a_opt = {
	    'data-toggle': 'tooltip',
	    'data-container': 'body',
	    'data-placement': data[i]['placement']  || 'bottom',
	    'data-html': 'true',
	    title: html
	};
	var a = $s('a').attr(a_opt);
	svg.append(a);
	var rect_opt = {
	    x: '' + current_x,
	    y: '0',
	    width: '' + w,
	    height: '' + height,
	    fill: symbol_color(data[i]['color'])
	};
	a.append($s('rect').attr(rect_opt));
	current_x += w;
    }
    return svg;
}

// QUERY_STRINGを考慮して現在のURLにパラメータを付与する
function append_link(append_opt) {
    var ret = location.href;
    if (ret.indexOf('?') >= 0) {
	ret += '&' + append_opt;
    } else {
	ret += '?' + append_opt;
    }
    return ret;
}

// Stacked Fill Graphで使用。yの値にbaseのyの値を加算して返す。
function calc_stacked(base, stack) {
    for (var i = 0; i < base['y'].length; i++) {
	stack['y'][i] += base['y'][i];
    }
    return stack;
}

// 指定した箇所の幅を取得し、グラフの幅を計算して返す。
function graph_width(target) {
    return $(target).innerWidth() - 10;
}

//
function url_replace_id(target, id) {
    return $(target).val().replace('___id___', '' + id);
}

// 指定されたidのホスト詳細画面のURLを返す。
function url_host_detail(id) {
    return url_replace_id('#sidfm_host_path', id);
}

// 指定されたidの脆弱性詳細画面のURLを返す。
function url_vulnerabilities_detail(id) {
    return url_replace_id('#sidfm_vulnerability_path', id);
}
function url_vulnerability_detail(id) {
    return url_replace_id('#sidfm_vulnerability_path', id);
}

//
function url_info_product_detail(id) {
    return url_replace_id('#sidfm_informations_product_path', id);
}

//
function url_info_vuln_detail(id) {
    return url_replace_id('#sidfm_informations_vuln_path', id);
}

// 指定した要素にclickに反応しないようにする。
// FIXME: 既にclickイベントが登録されている場合への対応
function force_click_ignore() {
    $('.ignore-click').on('click', function() {
	return false;
    });
}

// ソート処理
function sort_toggle(name, base = 'pages') {
    return (function() {
	var order = $('#' + base + '_order').val();
	if (order == name + '_asc') {
	    $('#' + base + '_order').val(name + '_desc');
	} else {
	    $('#' + base + '_order').val(name + '_asc');
	}
	$('#' + base + '_form').submit();
	return false;
    });
}

// URLやパスから、anchor (fragment) を削除する
function remove_anchor(str) {
    return (str ? str.replace(/#[0-9a-zA-z_-]+$/, '') : '');
}

// ヘッダのタイトルを更新する。
function header_title_update(title) {
    var header_title = $('#sidfm_header_title');
    var vm_page_title = $('#vm_page_title').text();
    header_title.text(vm_page_title);
    header_title.removeClass('d-none');
}

// windows.onerrorでアプリケーションにロギングする
// なお、ログイン状態でないとロギングに失敗する（仕様）
function logging_error() {
    window.onerror = function(msg, file, line, column, err) {
	if (!msg) { msg = '(no msg)'; }
	if (!file) { file = '(no file)'; }
	if (!line) { line = '(unknown)'; }
	if (!column) { column = 0; }
	if (!err) { err = null; }
	
	var logger_path = $('#sidfm_internal_api_logger_path').val();
	if (logger_path) {
	    $.post(logger_path,
		   { 'params[msg]': msg,
		     'params[file]': file,
		     'params[column]': column,
		     'params[line]': line,
		     'params[err]': err },
		   function(data) {
		       console.log('msg: ' + msg);
		       console.log('file: ' + file);
		       console.log('column: ' + column);
		       console.log('line: ' + line);
		       console.log('err: ' + err);
		       console.log('data:' + data);
		   });
	}
	return false;
    };
}

function sticky_control(table_name, page_name) {
  if( $('#wrap-ActionArea').length === 0 ){
      return false;
  }
  window.onscroll = function() { stickyAdd(); };
  var wrap_ActionArea = document.getElementById("wrap-ActionArea");
  var target = document.getElementById(table_name);
  const footer = $('#sidfm-footer');
  $('#ActionArea').css('margin-left','0');
  $('.modal').on('show.bs.modal', function(){
      setTimeout(function(){
          $('#wrap-ActionArea').css('z-index', '10');
      });
  });

  $('.modal').on('hidden.bs.modal', function(){
      if($('.modal:visible').length === 0){
          $(wrap_ActionArea).css('z-index', '1045');
      }else{
          $(wrap_ActionArea).css('z-index', '500');
      }
  });

  function stickyAdd() {
      var calc_offset = function(){
                            var offset = 0;
                            if(page_name === 'hosts' || page_name === 'host' || page_name === 'vulnerability' || page_name === 'rule'|| page_name === 'lproducts' || page_name === 'products' || page_name === 'local_vuls'){
                                var SummaryT  = document.getElementById('SummaryTable');
                                var SummaryTdynamic = SummaryT.offsetTop;
                                var SummaryTHeight= document.getElementById('SummaryTable').clientHeight;
                                offset = SummaryTdynamic + SummaryTHeight;
                             }else if(page_name === 'vulnerabilities'){
                                 var SummaryT  = document.getElementById('SummaryTable');
                                 var SummaryStaticOffset = SummaryT.offsetTop;
                                 var SummaryTHeight= document.getElementById('SummaryTable').clientHeight;
                                 var accordion1_h  = document.getElementById('system_category_table').clientHeight;
                                 var accordion2_h  = document.getElementById('OS_distributions').clientHeight;
                                 var accord_h = accordion1_h + accordion2_h;
                                 offset = SummaryStaticOffset + accord_h + SummaryTHeight;
                             }else if(page_name === 'users' || page_name === 'licence' || page_name === 'histories' || page_name === 'categories' || page_name === 'register_apikey' || page_name === 'ctags' || page_name === 'cves' || page_name === 'vm_apikey' || page_name === 'cat-groups'){
                                 var clientRect = target.getBoundingClientRect();
                                 offset = clientRect.top; 
                             }
                             return offset;
                         };
      if (window.pageYOffset >= calc_offset()) {
          if( $(wrap_ActionArea).find('footer').length === 0){
              $(wrap_ActionArea).append(footer);
          }
          $('#hidden-space').height($('#wrap-ActionArea').outerHeight());
          wrap_ActionArea.classList.add('sticky');
      } else {
          $('#hidden-space').height('0');
          wrap_ActionArea.classList.remove('sticky');
          $('#before-sidfm-footer').after(footer);
      }
  }
}

//アクションボタンとフィルターのdropdownがあるページで使用する。
//modalを開いた後にフィルターボタンを押下すると、
//アクションボタンと被る部分が表示されないため、この処理を入れる
function set_style_for_ActionArea_on_filter_dropdown_event(){
    $('.summary-filter-buttons [data-toggle="dropdown"]').parent().on('show.bs.dropdown', function(){
        $('#wrap-ActionArea').css('z-index', 10);
    });

    $('.summary-filter-buttons [data-toggle="dropdown"]').parent().on('hidden.bs.dropdown', function(){
        $('#wrap-ActionArea').css('z-index', 1045);
    });
}

function set_scroll(bindEventsNecessary = true){
    const wrapActionArea = $('#wrap-ActionArea');
    const ActionArea = $('#ActionArea');
    const floatingScrollId = 'floating-scroll';
    var ScrollTable = $('[data-outertable="scroll"]');
    var inner_table = $('[data-innertable="scroll"]');
    var mainContainer = $('#MainContainer');
 
    if( !mainContainer.length || !ScrollTable.length || !inner_table.length || !wrapActionArea.length ){
        return;
    }
    const pagenationArea = ScrollTable.find('.fixed-table-pagination');
    var tableBody = ScrollTable.find('.fixed-table-body');
    if( !pagenationArea.length || !tableBody.length ){
        return;
    }
    var ScrollTable_offset = ScrollTable.offset().top;
    var MainContainer_bottom = parseInt(mainContainer.css('padding-bottom').replace('px', ''));
    const tableScrollBarHeight = (tableBody.get(0).offsetHeight - tableBody.get(0).clientHeight); //スクロールバーの高さ
    var space_for_scroll =  (MainContainer_bottom > 0 ? MainContainer_bottom + tableScrollBarHeight : tableScrollBarHeight)
                            + parseInt(ActionArea.css('padding').replace('px', ''))
                            ;
    var doc_height = $(document).innerHeight();
    var window_height = $(window).innerHeight();
    var scrolled_quantity = $(window).scrollTop();
    var bottom_height = wrapActionArea.hasClass('sticky') ?
        pagenationArea.outerHeight(true) + tableScrollBarHeight :
        (ActionArea.outerHeight(true) + $('footer').outerHeight(true) + pagenationArea.outerHeight(true) + tableScrollBarHeight)
    ;
    if(
        (doc_height - window_height) > (scrolled_quantity + bottom_height) && 
        ((ScrollTable_offset - window_height + 15) < scrolled_quantity) &&
        (tableBody.width() < tableBody.get(0).scrollWidth) &&
        ($(`#${floatingScrollId}`).length === 0)
    ){
        var dummy_scroll = $('<div>').attr('id', floatingScrollId).css(
            {
                'left': ScrollTable.offset().left,
                'bottom': wrapActionArea.hasClass('sticky') ? wrapActionArea.height() : 0
            }
        );
        var inner_dummy_scroll = $('<div>').attr('id', 'inner-floating-scroll').css(
            { 'height':'20px', 'z-index':'1060' }
         );
         dummy_scroll.append(inner_dummy_scroll);
         $('#content-block').append(dummy_scroll);
         dummy_scroll.on('scroll', function(){
             tableBody.get(0).scrollLeft = dummy_scroll.get(0).scrollLeft;
         });
    }
    setTimeout(function(){
      //ここだけ非同期にしないと、フッターの表示・非表示の切り替え時にフッターがある時の位置に#floating-scrollが残ってしまう時がある。
        if(!wrapActionArea.hasClass('sticky')){
            $(`#${floatingScrollId}`).css('bottom', '0');
        }else{
            $(`#${floatingScrollId}`).css('bottom',  wrapActionArea.height());
        }
    },);
    if((doc_height - window_height) > (scrolled_quantity + bottom_height) 
        && ((ScrollTable_offset - window_height + 15) < scrolled_quantity)){
        $(`#${floatingScrollId}`).show();
    }else{
        $(`#${floatingScrollId}`).hide();
    }
    $(`#${floatingScrollId}`).offset({'left':ScrollTable.offset().left})
                             .width(ScrollTable.outerWidth());
    $('#inner-floating-scroll').width(inner_table.width());
    
    if( bindEventsNecessary ){
        $('.modal').on('show.bs.modal', function(){
            $(`#${floatingScrollId}`).css('z-index', '10');
        });
        $('.modal').on('hidden.bs.modal', function(){ 
            if($('[data-slidemodal]').length === 0){
                $(`#${floatingScrollId}`).css('z-index', '1060');
            }else{
                $(`#${floatingScrollId}`).css('z-index', '10');
            }
        });
        $(window).on('resize scroll', function(){
            set_scroll(false);
        });
    }
}

//ステータス一括変更をprogress barを使って表示
const executingProcessOfProgressBar = { executing: null };
function execute_process_with_progress_bar(target, conf_message, info, path, reloadPage = true, dynamicSummaryTable = null){
    (async () => {
        if( executingProcessOfProgressBar.executing ) return;
        executingProcessOfProgressBar.executing = true;
        if( dynamicSummaryTable ){
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction();
            if( roleOrCategoryError ) return;
        }
        if( conf_message && conf_message.length > 0 ){
            var conf = confirm(conf_message);
            if( !conf ){
                executingProcessOfProgressBar.executing = false;
                return false;
            }
        }
        executingProcessOfProgressBar.executing = false;
        var ids = JSON.stringify(info['ids']);
        var action = info['action'];
        var comment = info['comment'];
        var filter_category_id = info['category_id'];
        if( target === 'status' ){
            var prev_status = info['prev_status'];
            var new_status = info['new_status'];
        }else{
            var force = String(info['force']);
            var due_date = info['due_date'];
        }
        var percentage = 0;
        var total_vuls_size;
        var total_executed_vuls_size = 0;
        var shift_num;
        var err_message = '';
        var issue_ids = [];
        if( target === 'status' ){
            if( new_status == 0 ){
                alert('変更後のステータスを選択して下さい');
                return false;
            }
            if( prev_status == new_status ){
                alert('変更前と変更後のステータスが同じです');
                return false;
            }
        }
        var progress = $('<div>').attr('class', 'progress-area').append(
            $('<div>').attr('class', 'progress-area-label').text('処理実行中...'),
            $('<div>').attr('class', 'progress').append(
                $('<div>').attr({
                    'id': 'progress-bar',
                    'class': 'progress-bar progress-bar-striped',
                    'role': 'progressbar',
                    'aria-valuenow': '0',
                    'aria-valuemin': '0',
                    'aria-valuemax': '100'
                }).text('0%')
            )
        );
        var wrap_progress = $('<div>').attr('class', 'position-relative h-100 w-100').append(progress);
        var progress_window = $('<div>').attr('class', 'progress-window').append(progress);
        $('body').append(progress_window);
       
        var executed_vuls_size = 0;
        function exec_bulk_partial_update(target) {
            if( target === 'status' ){
                d = {
                    issue_ids: JSON.stringify(issue_ids),
                    shift_num: shift_num,
                    ids: ids,
                    prev_status: prev_status,
                    new_status: new_status,
                    comment: comment,
                    action_name: action,
                    category_id: filter_category_id
                };
            }else{
                d = {
                    issue_ids: JSON.stringify(issue_ids),
                    shift_num: shift_num,
                    ids: ids,
                    force: force,
                    due_date: due_date,
                    comment: comment,
                    action_name: action,
                    category_id: filter_category_id
                };
            }
            $.ajax({
                url: path,
                type: 'POST',
                data: d,
            }).done(function(data){
                executed_vuls_size = data['executed_vuls_size'];
                if( data['total_size'] != null){
                    total_vuls_size = data['total_size'];
                }
                if( data['shift_num'] ){
                    shift_num = data['shift_num'];
                }
                issue_ids = data['issue_ids'];
                if( data['err'] ){
                    if( data['err'] == 'invalid' ){
                        err_message = '不正な操作です。';
                    }else if( data['err'] = 'db_error' ){
                        err_message = total_vuls_size + '件中のうち、' + total_executed_vuls_size + '件の処理が完了し、' +
                                      data['err_vuls_size'] + '件でエラーが発生しました。\n' +
                                      (total_vuls_size - total_executed_vuls_size) + '件は未処理となっています。';
                    }
                }
    
                if(err_message.length > 0){
                     alert(err_message);
                     $('#pages_checked').val('[]');
                     $('#pages_all_checked').val('');
                     $('#pages_form').submit();
                     return false;
                }else if(total_vuls_size === 0){
                     if( reloadPage ){
                         if( target === 'status' ){
                             alert('変更の対象となるデータがありませんでした。画面をリロードします。');
                         }else{
                             alert('変更の対象となるデータがありませんでした。画面をリロードします。\n※全てのデータに対策予定日が設定されている可能性があります。対策予定日が設定されているデータは、「強制上書き」にチェックを入れないと、変更の対象となりません。\n※また、対応状況が「対策済」と「影響無」のデータも変更の対象となりませんので、ご注意ください。');
                        }
                        $('#progress-bar').attr('aria-valuenow', '0').css('width', '').text('0%');
                        $('#pages_checked').val('[]');
                        $('#pages_all_checked').val('');
                        $('#pages_form').submit();
                        return false;
                    }else{
                        if( target === 'status' ){
                             alert('変更の対象となるデータがありません。');
                         }else{
                             alert('変更の対象となるデータがありません。\n※全てのデータに対策予定日が設定されている可能性があります。対策予定日が設定されているデータは、「強制上書き」にチェックを入れないと、変更の対象となりません。\n※また、対応状況が「対策済」と「影響無」のデータも変更の対象となりませんので、ご注意ください。');
                        }
                        progress_window.remove();
                        $('.modal:visible').modal('hide'); 
                        return false;
                    }
                }
                total_executed_vuls_size += executed_vuls_size;
                percentage = (total_executed_vuls_size / total_vuls_size) * 100;
                if( percentage >= 99 && percentage < 100 ){
                    percentage = 99;
                }else{
                    percentage = Math.round(percentage);
                }
                $('#progress-bar').attr('aria-valuenow', percentage).css('width', percentage + '%').text(percentage + '%');
                if( percentage === 100 ){
                    if( reloadPage ){
                        setTimeout(function(){//probress barのbarが満たされるまで待つ
                            $('#pages_checked').val('[]');
                            $('#pages_all_checked').val('');
                            if( action && action === 'single' ){
                                $('#pages_form').append($('<input>').attr({'type': 'hidden', 'name': 'pages[is_update_one_row]'}).val('true'));
                            }
                            $('#pages_form').submit();
                        },800);
                    }else{
                        if( dynamicSummaryTable ){
                            dynamicSummaryTable.refreshTable();//.updateSummaryTableRows(info['ids']); //progress_windowとmodalを閉じる処理は、javascripts/sidfm_dynamic_summary_table.js#enableDynamicSummaryTableで行っているため、ここでは処理しない
                        }else{
                            progress_window.remove();
                            $('.modal:visible').modal('hide');
                        }
                    }
                }else{
                    exec_bulk_partial_update(target);
                }
            });
         }
         exec_bulk_partial_update(target);
    })();
}

//ポップアップが、ヘッダーかアクションボタンに重なる時に、表示を調整する
function setPopupZindex(item){
   if( !item || !item.offset() ) return;
   const itemOffsetTop = item.offset().top;
   const HEADnavi = $('#HEADnavi');
   const HEADnaviBottom = HEADnavi.offset().top + HEADnavi.outerHeight();
   const ActionArea = $('#wrap-ActionArea');
   const ActionAreaOffsetTop = ActionArea.offset().top;
   const itemHeight = item.outerHeight();
   if( HEADnaviBottom > itemOffsetTop && ActionAreaOffsetTop > itemOffsetTop + itemHeight ){
       item.css('z-index', +HEADnavi.css('z-index') - 1);
   }else if( ActionArea.hasClass('sticky') &&
       ActionAreaOffsetTop < itemOffsetTop + itemHeight &&
       (($(document).innerHeight() - $(window).innerHeight()) > ($(window).scrollTop() + (ActionAreaOffsetTop + ActionArea.outerHeight()) - (itemHeight + itemOffsetTop + 15)))
   ){  
       item.css('z-index', +ActionArea.css('z-index') - 1);
   }else{
       item.css('z-index', +ActionArea.css('z-index') + 1);
   } 
}

const isAdjustBalloonDropdownStyleInited = { inited: false };
//サマリー表のバルーンのdropdownの表示を調整
function adjust_balloon_dropdown_style(){
    if( isAdjustBalloonDropdownStyleInited.inited ) return false; //イベントハンドラが重複登録されるのを防ぐ
    var set_z_index = function(){
        var target_balloon_menu = $('.balloon-dropdown').find('.dropdown-menu.show');
        if( target_balloon_menu.length > 0 ){
            var set = false;
            var header = $('#HEADnavi');
            if( header.length > 0 ){
                if( (header.offset().top + header.outerHeight()) > target_balloon_menu.offset().top ){
                    target_balloon_menu.css('z-index', parseInt(header.css('z-index')) - 1);
                    set = true;
                }
            }
            var pagenationArea = $('#pagenationArea');
            var ActionArea = $('#wrap-ActionArea');
            if( !set && ActionArea.length > 0){
                if( pagenationArea.is(':visible') ){
                    if( pagenationArea.offset().top > ActionArea.offset().top && (target_balloon_menu.offset().top + target_balloon_menu.outerHeight() + 5) > ActionArea.offset().top ){
                        target_balloon_menu.css('z-index', parseInt(ActionArea.css('z-index')) - 1);
                        set = true;
                    }
                }else{
                    /*if( target_balloon_menu.closest('.balloon-dropdown').hasClass('balloon-up') ){
                        if( ActionArea.offset().top < (target_balloon_menu.offset().top + target_balloon_menu.outerHeight() + 7) ){
                            target_balloon_menu.css('z-index', parseInt(ActionArea.css('z-index')) - 1);
                            set = true;
                        }
                    }else{*/
                        if( ActionArea.hasClass('sticky') && (target_balloon_menu.offset().top + target_balloon_menu.outerHeight() + 5) > ActionArea.offset().top && (($(document).innerHeight() - $(window).innerHeight()) > ($(window).scrollTop() + 50)) ){ //サマリー表最下部あたりのバルーンは、アクションボタンに被っても上に表示する
                            /*const floatingScrollZindex = $('#floating-scroll:visible').css('z-index') || 0;
                            const ActionAreaZindex = ActionArea.css('z-index') || 0;
                            const targetZindex = (floatingScrollZindex > ActionAreaZindex) ?
                                floatingScrollZindex :
                                ActionAreaZindex;*/
                            target_balloon_menu.css('z-index', parseInt(ActionArea.css('z-index')) - 1);
                            set = true;
                        }
                    //}
                }
                if(!set) target_balloon_menu.css('z-index', parseInt(ActionArea.css('z-index')) + 1);
            }
        }
    };

    $(document).on('shown.bs.dropdown', '.balloon-dropdown', function(){
        setTimeout(function(){ //非同期にしないと.dropdown-menuのoffsetを正しく認識しない
            set_z_index();
        });
    });

    $(window).on('scroll', function(){
        set_z_index();
    });
    isAdjustBalloonDropdownStyleInited.inited = true;
}

//サマリー表の幅が小さい時、バルーンの大きさ(表示可能な余白)を返す
/*function getBalloonSpace(balloonTriggerBtn, summaryTable, isDropUp){
    let space = 0;
    if( isDropUp ){
        space = balloonTriggerBtn.offset().top - summaryTable.offset().top - 25;
    }else{
        space = summaryTable.outerHeight() - (balloonTriggerBtn.offset().top - summaryTable.offset().top);
    }
    return space;
}*/

var jqxhr_for_list = null
//ホスト名・カテゴリ名を表示するバルーンの表示
function enable_dropdown_balloon(summaryTableId = '', funcBeforeShow = null){
    $(summaryTableId.length ? `#${summaryTableId} .create-balloon` : '.create-balloon').on('click', function(){
        var trigger_btn = $(this);
        if( funcBeforeShow ){
            funcBeforeShow($(this));
        }
        var trigger_parent = trigger_btn.closest('.balloon-dropdown');
        var max_width = trigger_parent.find('.balloon-menu').css('max-width').replace('px', '');
        var is_space_of_right = $(window).width() - (trigger_btn.offset().left + trigger_btn.outerWidth()) > max_width;
        /*trigger_parent.removeClass('dropright dropleft').addClass(is_space_of_right ? 'dropright' : 'dropleft');
        trigger_parent.removeClass('balloon-right balloon-left')
            .addClass(is_space_of_right ? 'balloon-right' : 'balloon-left');*/
        trigger_parent.removeClass('dropright dropleft balloon-up balloon-right balloon-left')
                      .addClass(is_space_of_right ? 'dropright balloon-right' : 'dropleft balloon-left');
        /*if( summaryTableId.length ){
            const dataSize = trigger_parent.find('[data-datasize]').data('datasize');
            const totalTableRows = $(`#${summaryTableId} tr[data-index]`).length;
            const rowPosition = parseInt(trigger_btn.closest('tr[data-index]').attr('data-index'));
            const isDropUp = totalTableRows - rowPosition < dataSize;
            if( isDropUp ){
                trigger_parent.removeClass('dropright dropleft dropright dropleft balloon-right balloon-left')
                              .addClass('dropup balloon-up');
            }
        }*/
        trigger_btn.attr({
            'data-toggle': 'dropdown',
            'data-boundary': 'window',
            'aria-haspopup': 'true',
            'aria-expanded': 'false',
            'data-flip': 'false',
        }).dropdown();
    });
    adjust_balloon_dropdown_style();
}

//ホストが多い場合にhtmlの表示に時間がかかる可能性があるため、ajaxを使って表示する
function enable_hosts_list_balloon(dynamicSummaryTable){
    const balloonTriggerSelector = '.create-hosts-list-ballon';
    $(balloonTriggerSelector).on('click', function(){
        var trigger_btn = $(this);
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction();
            if( roleOrCategoryError ) return;
            if( trigger_btn.attr('aria-haspopup') === 'true' ){
                return false;
            }
            var path = $('#sidfm_internal_api_get_vul_hosts_path').val();
            var html_id = trigger_btn.data('htmlid');
            var filter_category_id = trigger_btn.data('filtercategoryid'); //脆弱性別状況画面以外の画面ではundefになる。
            var trigger_parent = trigger_btn.closest('.balloon-dropdown');
            //var menuItem = trigger_parent.find('[data-itemslist]');
            //menuItem.css('max-height', '');
            if( !$('#hosts-list-' + html_id).is(':visible') ){
                var max_width = trigger_parent.find('.balloon-menu').css('max-width').replace('px', '');
                var is_space_of_right = $(window).width() - (trigger_btn.offset().left + trigger_btn.outerWidth()) > max_width;
                trigger_parent.removeClass('dropright dropleft balloon-up balloon-right balloon-left')
                              .addClass(is_space_of_right ? 'dropright balloon-right' : 'dropleft balloon-left');
                if( jqxhr_for_list ){
                    jqxhr_for_list.abort();
                }
                jqxhr_for_list = $.post(
                    path,
                    { 'html_id': html_id, 'filter_category_id': filter_category_id },
                    function(data){
                        var hosts = data['hosts'];
                        var htmlid = data['html_id'];
                        if( hosts.length === 0 ){
                            alert('ホストが削除されたか、ユーザもしくはホストのカテゴリが変更されているため、 画面をリロードします。')
                            $('#pages_checked').val('[]');
                            $('#pages_all_checked').val('false');
                            $('#pages_form').submit(); 
                        }
                        var target = $('#hosts-list-' + htmlid);
                        $('#hosts-size-' + htmlid).text(hosts.length);
                        $('#hosts-first-' + htmlid).text(hosts[0]['display_name']);
                        target.html('');
                        var append_host_elm = function(host){
                            var host_id = host['sidfmd_host_id'];
                            var host_elm = $('<div>').attr({
                                'class': 'host-list-item list-group-item text-truncate g-max-width-12em'
                            }).append(
                                $('<span>').attr({
                                    'data-toggle': 'tooltip',
                                    'data-placement': 'right',
                                    'data-boundary': 'window',
                                    'data-html': 'true',
                                    'title': 'ホスト名: ' + host['hostname'] + '<br/>表示名: ' + (host['ext_name'] ? host['ext_name'] : '(未設定)'),
                                }).append(
                                    $('<a>').attr({
                                        'href': url_host_detail(host_id)    
                                    }).text(host['display_name'])
                                )
                            );
                            target.append(host_elm);
                            target.find('[data-toggle=tooltip]').tooltip();
                        };
                        hosts.slice(0, 15).forEach(function(host){//すぐに表示させるために先に15件要素を作成
                            append_host_elm(host);
                        });
                        /*if( summaryTableId.length ){
                            const totalTableRows = $(`#${summaryTableId} tr[data-index]`).length;
                            const rowPosition = parseInt(trigger_btn.closest('tr[data-index]').attr('data-index'));
                            //const maxHeight = menuItem.css('max-height').replace('px', '');
                            //const summaryTable = $(`#${summaryTableId}`);
                            //bootstrap-tableがある時に、ページネーションにdropdown-menuが被る場合は、
                            //dropdown-menuを上に表示する
                            //const space = getBalloonSpace(trigger_btn, summaryTable, isDropUp);
                            const isDropUp = totalTableRows - rowPosition < hosts.length;
                            if( isDropUp ){
                                trigger_parent.removeClass('dropright dropleft dropright dropleft balloon-right balloon-left')
                                              .addClass('dropup balloon-up');
                            }
                            //if( space < parseInt(maxHeight) ){
                            //    menuItem.css('max-height', space + 'px');
                            //} 
                        }*/
                        trigger_btn.attr({
                            'data-toggle': 'dropdown',
                            'data-boundary': 'window',
                            'aria-haspopup': 'true',
                            'aria-expanded': 'false',
                            'data-flip': 'false',
                        }).dropdown();
                        if( !$('#hosts-list-' + html_id).is(':visible') ){
                            //bootstrapでclickイベントを見ているのか、
                            //dropdown.('show')にするとdropdown-menuがテーブルの内側に表示され、
                            //dropdown-menuが見えなくなるため、
                            //click()で表示する
                            //(data-boundary="window"が効かない状態になるのを防ぐ)
                            trigger_btn.click();
                        }
                        setTimeout(function(){
                            hosts.slice(15).forEach(function(host){
                                append_host_elm(host);//残りを作成    
                            });
                        });
                    }
                );
            }
        })();
    });
    adjust_balloon_dropdown_style();  
}

//カテゴリが多い場合にもhtmlの表示に時間がかかる可能性があるため、ajaxを使って表示する
function enable_categories_list_balloon(page_name, dynamicSummaryTable, onClickFunc = null){
    const balloonTriggerSelector = '.create-categories-list-balloon';
    $(balloonTriggerSelector).on('click', function(e){
        var trigger_btn = $(this);
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction();
            if( roleOrCategoryError ) return;
            var path = $('#sidfm_internal_api_get_hosts_vuls_related_categories_path').val();
            var html_id = trigger_btn.data('htmlid');
            var host_id = trigger_btn.data('hostid');
            var summary_prior_category = $('#summary_prior_category').val();
            var filter_category_id = page_name === 'vulnerabilities' ? trigger_btn.data('filtercategoryid') : null;
            var trigger_parent = trigger_btn.closest('.balloon-dropdown');
            var max_width = trigger_parent.find('.balloon-menu').css('max-width').replace('px', '');
            var is_space_of_right = $(window).width() - (trigger_btn.offset().left + trigger_btn.outerWidth()) > max_width;
            trigger_parent.removeClass('dropright dropleft balloon-up balloon-right balloon-left')
                          .addClass(is_space_of_right ? 'dropright balloon-right' : 'dropleft balloon-left');
            //var menuItem = trigger_parent.find('[data-itemslist]');
            //menuItem.css('max-height', '');
            /*var is_append_needed = false;
            if( (html_id && !$('#categories-list-' + html_id).is(':visible')) ||
                (host_id && !$('#categories-list-' + host_id).is(':visible')) ){
                is_append_needed = true;
            }*/
            if( jqxhr_for_list ){
                jqxhr_for_list.abort();
            }
            jqxhr_for_list = $.post(
                path,
                {
                    'html_id': html_id,
                    'host_id': host_id,
                    'filter_category_id': filter_category_id,
                    'page_name': page_name,
                    'prior_category': summary_prior_category
                },
                function(data){
                    var categories = data['categories'];
                    if( categories.length === 0 ){
                        alert('データが変更されているため、画面をリロードします。')
                        $('#pages_checked').val('[]');
                        $('#pages_all_checked').val('false');
                        $('#pages_form').submit();
                    }
                    var htmlid = data['html_id'];
                    var hostid = data['host_id'];
                    var is_bind_needed = false;
                    var target_id = hostid ? hostid : htmlid; //脆弱性詳細ではhtmlidとhostidの両方があり、hostidがある要素が対象のため、hostidを先に評価する
                    var target = $('#categories-list-' + target_id);
                    target.find('[data-toggle="tooltip"]').tooltip('hide');
                    target.html('');
                    $('#categories-size-' + target_id).text(categories.length);
                    $('#categories-first-' + target_id).text(
                        categories[0]['system_category_name'] ?
                        categories[0]['system_category_name'] :
                        categories[0]['custom_category_name']
                    );
                    if( target ){
                        var append_category_elm = function(category){
                            let attr = {
                                'class': 'categories-list-item list-group-item text-truncate g-max-width-16em',
                                'href': category['url'],
                                'data-toggle': 'tooltip',
                                'data-title': category['desc'],
                                'data-placement': 'right',
                                'data-boundary': 'window'
                            }
                            if( category.id ){
                                attr['data-value'] = category.id;
                            }
                            const catElm = $('<a>').attr(attr).text(category['system_category_name'] ? category['system_category_name'] : category['custom_category_name']);
                            target.append(catElm);
                            if( onClickFunc ) {
                                catElm.on('click', function(e){
                                    e.preventDefault();
                                    onClickFunc($(this));
                                });
                            } 
                        };
                        categories.slice(0, 15).forEach(function(category){//すぐに表示されるように先に15件要素を作成
                            append_category_elm(category);
                        });
                        target.find('a').tooltip();
                        /*if( dynamicSummaryTable.summaryTableId.length ){
                            const totalTableRows = $(`#${dynamicSummaryTable.summaryTableId} tr[data-index]`).length;
                            const rowPosition = parseInt(trigger_btn.closest('tr[data-index]').attr('data-index'));
                            //const maxHeight = menuItem.css('max-height').replace('px', '');
                            //const summaryTable = $(`#${dynamicSummaryTable.summaryTableId}`);
                            //bootstrap-tableがある時に、ページネーションにdropdown-menuが被る場合は、
                            //dropdown-menuを上に表示する
                            //const space = getBalloonSpace(trigger_btn, summaryTable, isDropUp);
                            const isDropUp = totalTableRows - rowPosition < categories.length;
                            if( isDropUp ){
                                trigger_parent.removeClass('dropright dropleft dropright dropleft balloon-right balloon-left')
                                              .addClass('dropup balloon-up');
                            }
                            //if( space < parseInt(maxHeight) ){
                            //    menuItem.css('max-height', space + 'px');
                            //}
                        }*/
                        setTimeout(function(){
                            categories.slice(15).forEach(function(category){//残りを作成
                                append_category_elm(category);
                            });
                            target.find('a').tooltip();
                        });
                        if( trigger_btn.attr('aria-haspopup') === 'true' ) return false;
                        trigger_btn.attr({
                            'data-toggle': 'dropdown',
                            'data-boundary': 'window',
                            'aria-haspopup': 'true',
                            'aria-expanded': 'false',
                            'data-flip': 'false',
                        }).dropdown();
                        if( !target.is(':visible') ){
                            trigger_btn.click();
                        }
                    }
                }
            );      
        })();
    }); 
    adjust_balloon_dropdown_style();
}

//progress_barを作成
function get_progress_bar(method_type = 'post'){
    var progress = $('<div>').attr('class', 'progress-area').append(
        $('<div>').attr({'class': 'progress-area-label', 'id': 'progress-bar-label'}).text('処理実行中...'),
        $('<div>').attr('class', 'progress').append(
            $('<div>').attr({
                'id': 'progress-bar',
                'class': 'progress-bar progress-bar-striped' + (method_type === 'get' ? ' bg-success' : ''),
                'role': 'progressbar',
                'aria-valuenow': '0',
                'aria-valuemin': '0',
                'aria-valuemax': '100'
            }).text('0%')
        )
    );
    var progress_window = $('<div>').attr({'id': 'progress-window', 'class': 'progress-window'}).append(progress);
    return progress_window
}

//HostView, VulViewのupdate
function update_view(host_ids = [], html_ids = [], is_appended = false, dynamicSummaryTable = null){
    if( dynamicSummaryTable ) dynamicSummaryTable.closeVisibleModals();
    if( host_ids.length > 0 || html_ids.length > 0 ){
        if( !is_appended ){
            var progress_window = get_progress_bar();
            $('body').append(progress_window);
        }
        var path = $('#sidfm_internal_api_update_view_path').val();
        var total_size = host_ids.length + html_ids.length;
        var executed_size = 0;
        var execute_update_view = function(){
            $.ajax({
                type: 'POST',
                url: path,
                data: { 'host_ids': JSON.stringify(host_ids), 'html_ids': JSON.stringify(html_ids) }
            }).done(function(data){
                if( data['executed_size'] === 0 ){
                    if( dynamicSummaryTable ){
                        dynamicSummaryTable.refreshTable();
                    }else{
                        if( $('#pages_form').length > 0 ){
                            setTimeout(function(){//progress barが満たされるまで待つ
                                $('#pages_checked').val('[]');
                                $('#pages_all_checked').val('false');
                                $('#pages_form').submit();
                            },500);
                        }else{
                            location.reload();
                        }
                    }
                }else{
                    host_ids = data['unexecuted_host_ids'];
                    html_ids = data['unexecuted_html_ids'];
                    executed_size += data['executed_size'];
                    var percentage = (executed_size / total_size * 100);
                    if( percentage >= 99 && percentage < 100 ){
                        percentage = 99;
                    }else{
                        percentage = Math.round(percentage);
                    }
                    $('#progress-bar').attr('aria-valuenow', percentage)
                                      .css('width', percentage + '%')
                                      .text(percentage + '%');
                    execute_update_view();
                }
            });
        };

        execute_update_view();
    }else{
        if( dynamicSummaryTable ){
            dynamicSummaryTable.refreshTable();
        }else{
            if( $('#pages_form').length > 0 ){
                $('#pages_checked').val('[]');
                $('#pages_all_checked').val('false');
                $('#pages_form').submit();
            }else{
                location.reload();
            }
        }
    }
}

//cvsswをクリックした時にCVE IDの一覧を表示する
function enable_display_cve_ids(dynamicSummaryTable){
    function arrange_cvss_list_position(button, list_elm){
        if( !button ){
            button = $('.show-summary-cves[data-vulid="' + list_elm.attr('data-vulid') + '"]');
        }
        var offset = button.offset();
        list_elm.offset({
            'top': offset.top - 5, 
            'left': offset.left + button.outerWidth()
        }).show();
        $('[data-toggle="dropdown"]').dropdown();
    }

    function create_summary_cve_info_list(button, vul_id, cves){
        $('.summary-cveinfo-list').remove();//表示するのは１つのみ
        $('[data-toggle="dropdown"]').parent().dropdown('hide');
        var list_elm = $('<div>').attr({
            'class': 'summary-cveinfo-list',
            'data-vulid': vul_id
        });
        var content = $('<div>').attr('class', 'summary-cveids-list');
        content.append(
            $('<div>').attr('class', 'summary-cveids-list-title').text('CVE ID 一覧')//.text(cve_info['time'])//'CVE ID 一覧')
        );

        var cvss_v2_vectors = JSON.parse($('#cvss-v2-vectors').val());
        var cvss_v3_vectors = JSON.parse($('#cvss-v3-vectors').val());

        cves = cves.map(function(cve) {
            var display_cvss = '';
            var cvss_v3 = cve['cvss_v3'];
            if( cvss_v3 ){
                var cvss_text_elms = [];
                for( key in cvss_v3 ) {
                    var value = cvss_v3[key];
                    var target_item = cvss_v3_vectors[key];
                    var item_name = target_item['item_name'];
                    var value_name = target_item['values'][value];
                    cvss_text_elms.push(item_name + ': ' + value_name);
                }
                display_cvss += '<div class="summary-cvss-vs-title">CVSS V3</div>' +
                                '<div>' + cvss_text_elms.join('<br>') + '</div>';
            }

            var cvss_v2 = cve['cvss_v2'];
            if ( cvss_v2 ){
                var cvss_text_elms = [];
                for( key in cvss_v2 ) {
                    var value = cvss_v2[key];
                    var target_item = cvss_v2_vectors[key];
                    var item_name = target_item['item_name'];
                    var value_name = target_item['values'][value];
                    cvss_text_elms.push(item_name + ': ' + value_name);
                }
                display_cvss += '<div class="summary-cvss-vs-title">CVSS V2</div>' + 
                                '<div class="mb-1">' + cvss_text_elms.join('<br>') + '</div>';
            }

            return $('<div>').attr('class', 'd-flex summary-cve-item').append(
                       $('<div>').attr({
                           'class': 'summary-cveid'
                       }).text(cve['cve_id']),
                       $('<div>').attr({
                           'class': 'summary-cvss-vectors-area',
                       }).append(
                           $('<button>').attr({
                               'type': 'button',
                               'class': 'btn btn-xs summary-cvss-vectors-btn',
                               'data-toggle': 'tooltip',
                               'data-title': display_cvss.length > 0 ? display_cvss : '未登録',
                               'data-placement': 'right',
                               'data-html': 'true',
                               'data-boundary': 'window'
                           }).text('CVSS指標')
                       )
                   );
        });
        content.append(
            $('<div>').attr({
                'id': 'summary-cveids-list-body-' + vul_id,
                'class': 'summary-cveids-list-body',
            }).append(cves),
            $('<div>').attr('class', 'd-flex').append(
                $('<button>').attr({
                    'type': 'button',
                    'class': 'btn btn-primary ml-auto copybtn summary-cveids-list-copybtn copy-cveids-list',
                    'data-toggle': 'tooltip',
                    'data-title': 'コピーしました!',
                    'data-trigger': 'manual'
                }).text('コピー')//ClipboardJSでコピー。コピーの実装は各ページによって異なるため、各ページのjsで行う
            )
        );
        $('body').append(list_elm.append(content).hide());
        list_elm.find('[data-toggle="tooltip"]').tooltip();
        arrange_cvss_list_position(button, list_elm);
    }

    $(window).on('scroll resize', function(){
        $('.summary-cveinfo-list [data-toggle="tooltip"]').tooltip('hide');
        $('.summary-cveinfo-list').remove();
    });

    $(document).on('click', function(e){
        if( e.target ){
            var t_elm = $(e.target);
            if( !t_elm.hasClass('show-summary-cves') && !t_elm.hasClass('show-summary-cves-inner') && !t_elm.hasClass('copy-cveids-list') ){
                $('.summary-cveinfo-list [data-toggle="tooltip"]').tooltip('hide');
                $('.summary-cveinfo-list').remove();
            }
        }
    });

    $(document).on('mousedown', '[data-outertable], #floating-scroll', function(e){
        if( e.target ){
            if( !$(e.target).hasClass('show-summary-cves') && !$(e.target).hasClass('show-summary-cves-inner')){
                $('.summary-cveinfo-list [data-toggle="tooltip"]').tooltip('hide');
                $('.summary-cveinfo-list').remove();
            }
        }
    });
    
    $('[data-toggle="dropdown"]').parent().on('show.bs.dropdown', function(){
        $('.summary-cveinfo-list [data-toggle="tooltip"]').tooltip('hide'); 
        $('.summary-cveinfo-list').remove();
    });

    $('.filter-buttons-area .dropdown-menu').on('click', function(e){
        e.stopPropagation();
    });

    const FetchInfo = { isFetching: false };

    $('.show-summary-cves, .show-summary-cves-inner').on('mousedown', function(e){
        var button = $(this);
        if( FetchInfo.isFetching ) return;
        FetchInfo.isFetching = true;
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction();
            if( roleOrCategoryError ) return;
            if( button.hasClass('no-cves') ){
                return false;
            }
            if( button.hasClass('show-summary-cves-inner') ){
                button = button.closest('.show-summary-cves');
            }
            e.stopPropagation();
            var id = button.attr('data-vulid');
            var target = $('.summary-cveinfo-list[data-vulid="' + id + '"]');
            $('.summary-cveinfo-list [data-toggle="tooltip"]').tooltip('hide');
            if( target.length > 0 ){
                target.remove();
            }else{
                if( button.attr('data-getdata') ) {
                    if( button.data('pagename') === 'host' ){
                        var path = $('#get_vul_view_cve_info_of_host_path').val();
                        var param = { 'html_id': id, 'host_id': $('#host_id').val() };
                    }else{
                        var path = $('#sidfm_internal_api_get_vul_view_cve_info_path').val();
                        var param = { 'vul_id': id };
                    }
                    $.post(path, param, function(data){
                        if( data['err'] ){
                            alert('データが変更されている、もしくはデータの取得に失敗しました。画面をリロードします。');
                            location.reload(true);
                        }
                        create_summary_cve_info_list(button, id, data['cves']);
                    });
                }else{
                    var cves = JSON.parse(button.attr('data-cves'));
                    create_summary_cve_info_list(button, id, cves);
                }
                
            }
            FetchInfo.isFetching = false;
        })();
        return false;
    });
}

//ホスト詳細・脆弱性詳細のサマリー表で、影響するパッケージ・プロダクトの表示
//tableの縦幅が狭い場合に表示できないのを、表示できるように細工
function enable_affitems_dropdown(scroll_summary_table_id){
    $('#' + scroll_summary_table_id + ' .show-affitems').on('click', function(e){
        e.stopPropagation();
        e.preventDefault();
        var button = $(this);
        var item_name = button.data('affitem');
        var displaying_same_affitem = $('.affitems-menu-show[data-affitem="' + item_name + '"]');
        if( displaying_same_affitem.length > 0 ){
            displaying_same_affitem.remove();
            return false;
        }
        displaying_same_affitem.remove();
        var clone_d_menu = button.closest('.dropdown')
                                 .find('.dropdown-menu')
                                 .clone()
                                 .addClass('affitems-menu-show')
                                 .attr('data-affitem', item_name);
        $('body').append(clone_d_menu);
        var target_offset = button.offset();
        
        var ActionArea_top = $('#wrap-ActionArea').offset().top;
        clone_d_menu.offset({
          'top': (ActionArea_top < (target_offset.top + clone_d_menu.outerHeight()) ? ActionArea_top - clone_d_menu.outerHeight() : target_offset.top),
          'left': target_offset.left - clone_d_menu.outerWidth(true)
        }).show();
        clone_d_menu.find('[data-toggle="tooltip"]').tooltip();
    });

    $('#' + scroll_summary_table_id + ' .show-affitems-dropdown-btn').parent().on('hide.bs.dropdown', function(){
        $('.affitems-menu-show').remove();
    });
 
    var remove_affitems_menu = function(){
        $('.affitems-menu-show').remove();
        $('.show-affitems-dropdown-btn').parent().dropdown('hide');
    };

    $(window).on('scroll', function(){
        remove_affitems_menu();
    });

    $('#' + scroll_summary_table_id).on('scroll', function(){
        remove_affitems_menu();
    });
}

//JSON/TSVのプログレスバーを使ったサマリー表のデータダウンロード中止
function cancel_json_tsv_download(message){
    alert(message + '画面をリロードします。');
    //$('#pages_checked').val('[]');
    //$('#pages_all_checked').val('false');
    $('#pages_form').submit();
}

//JSON/TSVのプログレスバーを使ったサマリー表のデータダウンロード
function json_tsv_summary_table_progress_download(target_format, path, opts = {}){
    (async () => {
        var conf = confirm('ファイルをダウンロードします');
        if( !conf ){
            return false;
        }
        if( opts.dynamicSummaryTable ){
            const roleOrCategoryError = await opts.dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                additionalAlertMessage: 'ファイルのダウンロードに失敗しました。'
            });
            if( roleOrCategoryError ) return;
        }
        $.cookie('export_flg',"",{path:"/",expires:-1});
        var progress_window = get_progress_bar('get');
        progress_window.find('#progress-bar-label').text('お待ちください...');
        $('body').append(progress_window);
        var order = $('#pages_order').val();
        var filters = $('#pages_filters').val();
        var category_id = $('#pages_category_id').val();
        var idx = $('#pages_idx').val();
        var executed_size = 0;
        var total_size = 0;
        var percentage = 0;
        var ids = JSON.parse($('#pages_checked').val());
        var process = 'init';
        var read_data = function(){
            $.ajax({
                url: path,
                type: 'POST',
                data: {
                    'target': target_format,
                    'ids': JSON.stringify(ids),
                    'executed_size': executed_size,
                    'process': process,
                    'order': order,
                    'filters': filters,
                    'category_id': category_id,
                    'idx': idx
                }
            }).done(function(res){
                //console.log(res)
                //res.idsは、ホスト詳細画面のパッケージ情報出力(のみ)で、idの配列ではなくパッケージ名の配列が返ってくることに注意
                if( res.err_message && res.err_message.length > 0 ){
                    cancel_json_tsv_download(res.err_message);
                }else{
                    if( process === 'init' ){
                        if( res.ids.length > res.max_download_size ){
                            var conf = confirm('ダウンロード可能な最大件数を超えていますが、最大件数まではダウンロード可能です。 \nダウンロードしますか?');
                            if( conf ){
                                //res.process_idsは、単純にsliceで処理できない際に、処理するidを指定する時に使う
                                res.ids = (res.process_ids ? res.process_ids : res.ids.slice(0, res.max_download_size))
                            }else{
                                $('#progress-window').remove();
                                return false;
                            }
                        }else{
                            if( res.process_ids ){
                                res.ids = res.process_ids; //res.process_idsの指定があれば、そちらを処理対象にする
                            }
                        }
                        $('#progress-bar-label').text('処理実行中...');
                        //始めの応答ではデータ数を取得し、processを書き換える。
                        total_size = res.ids.length;
                        process = 'progress_download';
                    }
                    if( res.ids.length > 0 ){
                        executed_size = res.executed_size;
                        ids = res.ids;
                        percentage = (executed_size / total_size * 100);
                        if( percentage >= 99 && percentage < 100 ){
                             percentage = 99;
                        }else{
                             percentage = Math.round(percentage);
                        }
                        $('#progress-bar').attr('aria-valuenow', percentage).css('width', (percentage < 3 ? 3 : percentage) + '%').text(percentage + '%');
                        read_data();
                    }else{
                        percentage = 100;
                        $('#progress-bar').attr('aria-valuenow', percentage).css('width', percentage + '%').text(percentage + '%');
                        if( opts['form'] ){
                            opts['form'].submit();
                        }else{
                            $('#form-download').submit();
                        }
                        $('#progress-bar-label').text('ファイル送信中...');
                        var timer = setInterval(function(){
                            if( $.cookie('export_flg') ){
                                setTimeout(function(){//probress barのbarが満たされるまで待つ
                                    $.cookie('export_flg',"",{path:"/",expires:-1});
                                    $('#progress-window').remove();
                                },500);
                                clearInterval(timer);
                            }
                        },10);
                    }
                }
            }).fail(function(){
                cancel_json_tsv_download('ファイルのダウンロードに失敗しました。');
            });
        };
    
        read_data();
    })();
}

//pages_checked, pages_all_checkedのリセット
function reset_pages_checked_values(){
    $('#pages_all_checked').val('');
    $('#pages_checked').val(JSON.stringify([]));
}

function sanitize_string(str){
    return str.replace(/\\/g, '\\\\').replace(/'/g, '\\\'');
}

function escape_html_special_chars(str){
    return str.replace(/\"/g, '&quot;').replace(/\'/g, '&#39;').replace(/</g, '&lt;').replace(/>/g, '&gt;');
}

//関連するルールを表示
function display_related_rules(data, show_modal = true){
    $('#RuleInfoArea > *').remove();
    var rule_messages = data['rule_messages'];
    var rule_ids = Object.keys(rule_messages);
    rule_ids.sort(function(a,b){
        if( rule_messages[a]['rule_name'] < rule_messages[b]['rule_name'] ){
            return -1;
        }else{
            return 1;
        }
    });
    var append_row = function(rule_id){
        var rule_message = rule_messages[rule_id];
        var rule_name = rule_message['rule_name'];
        var item_name = rule_message['item_name'];
        var content = rule_message['content'];
        var reason = rule_message['reason'];
        var tr = $('<tr>').attr('class', 'list_alternative');
        var rule_name_td = $('<td>').attr('class', 'rule-name').append($('<div>').attr('class', 'scrollable-menu-30vh').text(rule_name));
        var item_td = $('<td>').attr('class', 'rule-item').append($('<div>').attr('class', 'scrollable-menu-30vh').text(item_name));
        var message_td = $('<td>').attr('class', 'rule-changes').append($('<div>').attr('class', 'scrollable-menu-30vh').text(content));
        var reason_td = $('<td>').attr('class', 'rule-reason').append($('<div>').attr('class', 'scrollable-menu-30vh').text(reason));
        tr.append(rule_name_td, item_td, message_td, reason_td);
        $('#RuleInfoArea').append(tr);
    }
    var num = 15; //すぐに表示されるように、はじめだけ表示分をappend
    rule_ids.slice(0, num).forEach(function(rule_id){
        append_row(rule_id);
    });

    if( show_modal ){
        $('#modalConfRelatedRules').modal('show');
    }

    setTimeout(function(){
        rule_ids.slice(num, rule_ids.length).forEach(function(rule_id){
            append_row(rule_id);
        });
    });
}

function cancel_process_related_to_rules(redirect_url = null){
    alert('処理の実行前にデータが変更された可能性があるため、処理を中断しました。\nデータの変更の有無を確認後、操作をやり直してください。');
    if( redirect_url ) {
        location.href = redirect_url;
    }else{
        $('#pages_form').submit();
    }
    return;
}

function create_loading_window(message = 'お待ちください...'){
    return $('<div>').attr({'id': 'checking-window', 'class': 'progress-window'}).append(
               $('<div>').attr('class', 'progress-wait-area').append(
                   $('<div>').attr({'class': 'progress-wait-area-label'}).text(message),
               )
           );
}

//文字列から、日付のみを取得する
//日付として読み取れるなら日付の文字列が、文字数が0なら空文字が、日付でないか取扱いできない日付ならfalseを返す
function get_date(str_date){
    if( str_date.length === 0 ){
        return '';
    }
    str_date = str_date.replace(/年/, '/').replace(/月/, '/').replace(/日/, '/');
    if( !str_date.match(/[0-9]{4}[\/|\-]([0-9]|0[0-9]|1[0-2])[\/|\-]([0-9]|0[0-9]|1[0-2])/) ){
        return false;
    }

    var is_date = true;
    try{
        var date = new Date(str_date);
    }catch{
        is_date = false;
    }
    if( !is_date ){
        return false;
    }
    var year = date.getFullYear();
    var month = date.getMonth() + 1;
    var day = date.getDate();
    if( !(year + '/' + month + '/' + day).match(/[0-9]{4}\/([0-9]|0[0-9]|1[0-2])\/([0-9]|0[0-9]|1[0-2])/) ){
        return false;
    }
    if( String(month).length === 1 ){
        month = '0' + month;
    }
    if( String(day).length === 1 ){
        day = '0' + day; 
    }
    return year + '/' + month + '/' + day;
}

//ホスト別状況画面等の、画面上部にあるカテゴリのフィルタの操作
function enable_filter_main_category_menu(){
    var target_tab_content = ($.cookie('category_tag') === 'custom_category' ? '#filter-main-category-select-custom-category-tab-content' : '#filter-main-category-select-system-category-tab-content');
    $('#selected-main-category li [data-target="' + target_tab_content + '"]').addClass('active');
    $(target_tab_content).show();

    $('#filter-main-category-select-tabs-area .nav-link').on('click', function(e){
        var tab_item = $(this);
        if( !tab_item.hasClass('active') ){
            var target_content_id = tab_item.data('target');
            $('#filter-main-category-select-tabs-area .nav-link, #filter-main-category-select-tabs-area .nav-item').removeClass('active');
            tab_item.addClass('active');
            tab_item.closest('.nav-item').addClass('active');
            $('#filter-main-category-tab-content [data-tabcontent="true"]').hide();
            $(target_content_id).show();
            $.cookie('category_tag', (target_content_id.indexOf('system-category') === -1 ? 'custom_category' : 'system_category'), { path:'/' });
        }
        e.stopPropagation();
    });
}

//ホスト別状況画面等にある、ページ上部のカテゴリのフィルタを保存
//SidfmControllerSupport#get_filtering_main_category_idで値を取得する
function set_filtering_main_category_id(page_name, category_id){
    var target = 'sidfm_' + page_name + '_main_category_id';
    $.cookie(target, category_id);
}

function reloadOnFail(message){
    alert(message);
    $('#pages_all_checked').val('');
    $('#pages_checked').val('[]');
    const form = $('#pages_form');
    if( form.length > 0 ){
        form.submit();
    }else{
        location.reload(true);
    }
    return false;
}

//影響のある製品情報をvuln AP1Iから(もしくはローカル脆弱性から)取得して表示する
function enableDisplayVulnAffectProducts(getVulnPath, getLocalVulnAffectProductPath){
    const copyBtnClass = 'copy-affect-products';
    const clipboard_supported = ClipboardJS.isSupported();
    const clipboard = new ClipboardJS('.' + copyBtnClass);
    $(document).on('click', '.' + copyBtnClass, function(e){
        e.stopPropagation();
        if(clipboard_supported === false) {
            alert('このブラウザではコピー機能はサポートされておりません');
            return false;
        }
    }); 
    clipboard.on('success', function(e){
        var obj = $(e.trigger);
        obj.tooltip('show');
        setTimeout(function (){
            obj.tooltip('hide');
            e.clearSelection();
        }, 1000);
    });
    clipboard.on('error', function(e) {
        alert('コピーに失敗しました');
        e.clearSelection();
    });

    const getSidfmProductNames = (htmlid) => {
        return new Promise((resolve, reject) => {
           $.ajax({
                type: 'POST',
                url: getVulnPath,
                data: { id: htmlid }
            }).done(function(res){
                productNames = [];
                vuln = res.vuln;
                if( vuln ){
                    if( vuln.affect_products ){
                        vuln.affect_products.forEach( item => {
                            if( !item.product || !item.product.name ) return;
                            productNames.push(item.product.name);
                        });
                    }
                }
                resolve(productNames);
            }).fail(function(){
                reloadOnFail('エラーが発生しました。');
                reject(false);
            });
        });
    };

    const getLocalVulnProductNames = (htmlid) => {
        return new Promise((resolve, reject) => {
            $.ajax({
                type: 'POST',
                url: getLocalVulnAffectProductPath,
                data: { 'htmlid': htmlid }
            }).done(function(res){
                productNames = res;
                resolve(productNames);
            }).fail(function(){
                reloadOnFail('エラーが発生しました。');
                reject(false);
            });
        });
    };

    const createProductList = (productNames, htmlid) => {
        const copyElmId = 'affect-products-info-content-' + htmlid;
        const container = $('<div>').attr('class', 'affect-products-info-container');
        if( productNames.length ){
            const content = $('<ul>').attr({
                'id': copyElmId,
                'class': 'affect-products-info-content mb-0'
            });
            content.append(productNames.map( name => $('<div>').text(name) ) );
            const copyBtn = $('<button>').attr({
                'type': 'button',
                'class': `${copyBtnClass} g-font-size-0_74x btn btn-success btn-xs text-nowrap ml-auto`,
                'data-toggle': 'tooltip',
                'data-title': 'コピーしました !',
                'data-trigger': 'manual',
                'data-placement': 'bottom',
                'data-clipboard-target': '#' + copyElmId
            }).text('コピー');
            const copyBtnArea = $('<div>').attr('class', 'text-center affect-products-info-copy-btn-area').append(
                copyBtn
            );
            container.append(content, copyBtnArea);
        }else{
            container.append($('<div>').attr('class', 'affect-products-info-no-products').text('影響するプロダクトはありません'));
        }
        return $('<div>').append(container);
    };
    const displayClass = 'displaying';
    const displayContent = (button) => {
        button.tooltip('show').addClass(displayClass);
        button.find('[data-toggle="tooltip"]').tooltip(); 
    };
    $('.show-affect-products-info').on('click', function() {
        const button = $(this);
        const loading = button.find('.show-affect-products-info-loading');
        if( button.hasClass(displayClass) ){
            button.removeClass(displayClass);
            button.tooltip('hide');
            return;
        }
        $('.show-affect-products-info').tooltip('hide').removeClass(displayClass);
        if( button.attr('data-original-title') ){
            displayContent(button);
            return;
        }
        if( loading.is(':visible') ){
            return;
        }
        loading.show();
        const isLocalVul = button.data('local');
        const getProdNames = async () => {
            const htmlid = button.data('htmlid');
            let ProductNames = [];
            if( isLocalVul ) {
                ProductNames = await getLocalVulnProductNames(htmlid);
            }else{
                ProductNames = await getSidfmProductNames(htmlid);
            }
            button.attr('data-original-title', createProductList(productNames).html());
            displayContent(button);
            button.on('hide.bs.tooltip', function(){
                $(`.${copyBtnClass}[data-toggle="tooltip"]`).tooltip('hide');
            });
            loading.hide();
        };
        getProdNames();
    });
}

//脆弱性詳細画面のタブの内容を表示する
function enableDisplayVulnInformationContent(getVulnPath, getLocalVulnPath, getMsVulnPath, dynamicSummaryTable){
    const items = [
        { 'title': '概要', 'item': 'description' },
        { 'title': '影響を受ける製品', 'item': 'affect', 'active': true },
        { 'title': '影響などの確認方法', 'item': 'verification' },
        { 'title': '対処方法', 'item': 'solution' },
        { 'title': '関連情報', 'item': 'link' },  
        { 'title': '更新履歴', 'item': 'history' }
    ];

    const FetchInfo = { isFetching: false };
       
    const createTab = (item) => {
        return $('<div>').attr({
                   'class': 'show-content nav-tab-item' + (item.active ? ' active' : ''),
                   'data-target': item.item
               }).append(
                   $('<div>').attr('class', 'text-center').text(item.title)
               );
    };
    const createSelectArea = (roundTop = true) => {
        const tabArea = $('<div>').attr('class', 'content-info-select-area' + (roundTop ? ' round-border-top-1' : ''));
        const tabs1 = $('<div>').attr('class', 'nav-tab-list').append(
            items.slice(0, 3).map( item => createTab(item) )
        );
        const tabs2 = $('<div>').attr('class', 'nav-tab-list').append(
            items.slice(3).map( item => createTab(item) )
        );
        tabArea.append(tabs1, tabs2);
        return tabArea;
    };
    const getSidfmVulnContent = (htmlid) => {
        return new Promise((resolve, reject) => {
           $.ajax({
                type: 'POST',
                url: getVulnPath,
                data: { id: htmlid }
            }).done(function(res){
                if( res.vuln ){
                    resolve(res.vuln.content);
                }else{
                    resolve(null);
                }
            }).fail(function(){
                reloadOnFail('エラーが発生しました。');
                reject(false);
            });
        });
    };

    const getLocalVulnContent = (htmlid) => {
        return new Promise((resolve, reject) => {
            $.ajax({
                type: 'POST',
                url: getLocalVulnPath,
                data: { 'htmlid': htmlid }
            }).done(function(res){
                resolve(res.content);
            }).fail(function(){
                reloadOnFail('エラーが発生しました。');
                reject(false);
            });
        });
    };

    const getMsVulns = (htmlid) => {
        return new Promise((resolve, reject) => {
            $.ajax({
                type: 'GET',
                dataType: 'json',
                url: getMsVulnPath,
                data: { 'params[kb]': -htmlid }
            }).done(function(res){
                resolve(res.vulns);
            }).fail(function(){
                reloadOnFail('エラーが発生しました。');
                reject(false);
            });
        });
    };

    const createLoadingMark = (additionalClass = '') => {
        return $('<div>').attr('class', 'spinner-border text-light loading-selected-content-mark show-loading-content' + (additionalClass.length ? ` ${additionalClass}` : '')).append(
            $('<span>').attr('class', 'sr-only').text('Loading...')
        ).hide() 
    };

    const createMsVulnLinkContent = (msVulns) => {
        const len = msVulns.length;
        return msVulns.map( (vuln, index) => {
            return $('<div>').attr('class', 'p-1' + (index === len - 1 ? '' : ' mb-1')).append(
                $('<a>').attr({
                    'href': url_info_vuln_detail(vuln.id),
                    'class': 'word-break-all white-space-pre-wrap',
                    'target': '_blank',
                    'title': '別ウィンドウが開きます'
                }).text(vuln.id + ' ' + vuln.title)
            )
        });
    };

    const createContentDetailArea = (content) => {
        const area = $('<div>').attr('class', 'content-info-detail-area');
        area.append(
            items.map( item => {
                const elm = $('<div>').attr({
                    'class': 'content-info-detail',
                    'data-content': item.item
                }).html(content[item.item]);
                if( !item.active ){
                    elm.hide();
                }
                return elm;
            })
        );
        return area;
    };

    const createSelectVulnArea = (vulns) => {
       return $('<div>').attr('class', 'display-content-select-area round-border-top-1').append(
           $('<select>').attr('class', 'select-display-content display-content-select').append(
               vulns.map( vuln => $('<option>').text(`${vuln.id} ${vuln.title}`).val(vuln.id) )
           ),
           createLoadingMark()
       ); 
    };

    const createMsUpdateCatalogLink = (htmlid) => {
        return $('<div>').attr('class', 'no-content text-center pb-1').append(
            $('<a>').attr({
                'href': ('https://www.catalog.update.microsoft.com/Search.aspx?q=KB' + -htmlid),
                'target': 'blank',
                'title': '別ウィンドウが開きます'
            }).text('Microsoft Update カタログへ')
        );
    }

    const noInfo = $('<div>').attr('class', 'no-content text-center pb-1').text('詳細情報はありません。');
    const createDisplayContent = (content, htmlid, optionalVulns = []) => {
        const container = $('<div>').attr({ 'class': 'content-info-container', 'data-container': 'true' });
        container.append(createLoadingMark('top-50'));
        if( content ){
            const selectOptionNecessary = optionalVulns.length > 1;
            if( selectOptionNecessary ){
                //脆弱性コンテンツのデータが2つ以上ある場合はプルダウンを付与
                container.append(createSelectVulnArea(optionalVulns));
            }
            container.append(createSelectArea(!selectOptionNecessary), createContentDetailArea(content));
        }else{
            let displayContent = null;
            if( htmlid < 0 ){
                displayContent = createMsUpdateCatalogLink(htmlid);
            }else{
                displayContent = noInfo;
            }
            container.append(displayContent);
        }
        const closeBtn = $('<div>').attr('class', 'text-center close-content-btn-area').append(
            $('<button>').attr({
                'type': 'button',
                'class': 'btn btn-secondary close-content btn-mini',
                'data-htmlid': htmlid
            }).text('閉じる')
        );
        container.append(closeBtn);
        return $('<div>').append(container);
    };

    const displayClass = 'displaying';
    const displayContent = (button) => {
        button.tooltip('show').addClass(displayClass);
        button.find('[data-toggle="tooltip"]').tooltip();
    };
    const closeContent = (button) => {
        button.removeClass(displayClass);
        button.tooltip('hide');
    };

    const getContent = async (htmlid, isLocalVul) => {
        const optionalVulns = [];
        let content = null;
        if( htmlid < 0 ){
            const msVulns = await getMsVulns(htmlid);
            if( msVulns && msVulns.length > 0 ){
                const vuln = msVulns[0];
                if( vuln && vuln.id){
                    content = await getSidfmVulnContent(vuln.id);
                    optionalVulns.push(...msVulns);
                }
            }
        }else{
            if( isLocalVul ) {
                content = await getLocalVulnContent(htmlid);
            }else{
                content = await getSidfmVulnContent(htmlid);
            }
        }
        return { content, optionalVulns };
    };

    $('.show-information-content').on('click', function() {
        const button = $(this);
        
        if( button.hasClass(displayClass) ){
            closeContent(button);
            return;
        }
        $('.show-information-content').tooltip('hide').removeClass(displayClass);
        if( button.attr('data-original-title') ){
            displayContent(button);
            return;
        }
        const loading = button.find('.show-information-content-loading');
        if( loading.is(':visible') ){
            return;
        }
        const htmlid = button.data('htmlid');
        const isLocalVul = button.data('local');
        if( FetchInfo.isFetching ) return;
        FetchInfo.isFetching = true;
        const execDisplayContent = async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction();
            if( roleOrCategoryError ) return;
            loading.show();
            const { content, optionalVulns } = await getContent(htmlid, isLocalVul);
            button.attr('data-original-title', createDisplayContent(content, htmlid, optionalVulns).html());
            displayContent(button);
            loading.hide();
            FetchInfo.isFetching = false;
        };
        execDisplayContent();
    });

    $(document).on('click', '[data-container] .close-content', function(){
        const htmlid = $(this).attr('data-htmlid');
        const trigger = $('.show-information-content[data-htmlid="' + htmlid + '"]');
        if( trigger.attr('data-toggle') === 'tooltip'){
            closeContent(trigger);
        }
    });

    $(document).on('click', '.show-content[data-target]', function(){
        const elmParent = $(this).closest('[data-container]');
        if( elmParent.length && !$(this).hasClass('active')){
            elmParent.find('.show-content.active').removeClass('active');
            $(this).addClass('active');
            elmParent.find('[data-content]').hide();
            elmParent.find('[data-content="' + $(this).attr('data-target') + '"]').show();
        }
    });

    $(document).on('change', '[data-container] .select-display-content', function(){
        const select = $(this);
        const elmParent = select.closest('[data-container]');
        if( elmParent.length ){
            const loadingClass = 'pointer-events-none';
            select.addClass(loadingClass);
            elmParent.find('[data-content]').addClass(loadingClass);
            const loading = elmParent.find('.show-loading-content');
            loading.show();
            const htmlid = parseInt(select.val());
            const isLocalVul = false;
            const opt = select.find('option:selected');
            if( FetchInfo.isFetching ) return;
            FetchInfo.isFetching = true;
            const execDisplayContent = async () => {
                const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction();
                if( roleOrCategoryError ) return;
                const vuln = opt.data('vuln');
                var content;
                if( vuln ){
                    content = vuln;
                }else{
                    var { content } = await getContent(htmlid, isLocalVul);
                    opt.attr('data-vuln', JSON.stringify(content));
                }
                items.forEach( item => {
                    elmParent.find('[data-content="' + item.item + '"]')
                             .html(content[item.item])
                             .removeClass(loadingClass)
                             .scrollTop(0);
                })
                loading.hide();
                select.removeClass('pointer-events-none');
                FetchInfo.isFetching = false;
            };
            execDisplayContent();
        }
    });

    $(window).on('resize' ,function(){
        $('.show-information-content[data-toggle="tooltip"]').tooltip('hide');
    });
}

//カスタムタグの表示
function enableCtagListBalloon({
    dynamicSummaryTable,
    getCtagsPath = $('#sidfm_internal_api_get_ctags_by_htmlid_path').val(),
})
{
    const createCtagElm = (ctag) => {
        const styleProps = [];
        if( ctag.bgcolor ){
            styleProps.push('background-color: ' + ctag.bgcolor + ';');
        }
        styleProps.push('color: ' + (ctag.color ? ctag.color : '#212529') + ';');
        return $('<div>').attr({
            class: 'list-group-item text-truncate g-max-width-16em',
            style: styleProps.join(' '),
            'data-toggle': 'tooltip',
            'data-title': escape_html_special_chars(ctag.name) + '<br>カテゴリ: ' + escape_html_special_chars(ctag.category_name ? ctag.category_name : '全体'),
            'data-html': 'true',
            'data-placement': 'right',
            'data-boundary': 'window',
        }).text(ctag.name);
    };

    const initNum = 15;
    let jqxhr = null;
    $('.create-ctags-list-balloon').on('click', function(){
        const button = $(this);
        const sidfmdHostId = button.attr('data-hostid');
        const hostsCategoryId = button.attr('data-hostscategoryid');
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction();
            if( roleOrCategoryError ) return;
            if( button.attr('aria-haspopup') === 'true' ){
                return false;
            }
            const id = button.attr('data-id');
            const htmlid = button.attr('data-htmlid');
            const list = $('#ctags-list-' + id);
            var triggerParent = button.closest('.balloon-dropdown');
            const maxWidth = triggerParent.find('.balloon-menu').css('max-width').replace('px', '');
            const isSpaceOfRight = $(window).width() - (button.offset().left + button.outerWidth()) > maxWidth;
                triggerParent.removeClass('dropright dropleft balloon-up balloon-right balloon-left')
                             .addClass(isSpaceOfRight ? 'dropright balloon-right' : 'dropleft balloon-left');
            if( jqxhr ){
                jqxhr.abort();
            }
            const paramsData = { id: htmlid };
            if( sidfmdHostId ){
                paramsData.sidfmd_host_id = sidfmdHostId;
            }
            if( hostsCategoryId ){
                paramsData.hosts_category_id = hostsCategoryId;
            }
            jqxhr = $.post(
                getCtagsPath,
                paramsData,
                function(ctags){
                    const priorCtag = ctags[0];
                    //const displayingFirstCtag = $('#ctags-first-' + id);
                    if( !priorCtag ){
                        alert('カスタムタグが削除されているため、画面をリロードします。');
                        $('#pages_checked').val('[]');
                        $('#pages_all_checked').val('false');
                        $('#pages_form').submit();
                        return;
                    }
                    list.html('');
                    list.append(
                        ctags.slice(0, initNum)
                             .map(ctag => createCtagElm(ctag))//すぐに表示させるために先に15件要素を作成
                    );
                    list.find('[data-toggle="tooltip"]').tooltip();
                    button.attr({
                        'data-toggle': 'dropdown',
                        'data-boundary': 'window',
                        'aria-haspopup': 'true',
                        'aria-expanded': 'false',
                        'data-flip': 'false',
                    }).dropdown();
                    if( !list.is(':visible') ) button.click(); //clickで表示しないと、表示件数が少ない時にサマリー表内に表示されてしまい、ダイアログ全体が表示できない
                    setTimeout(function(){
                        list.append(
                            ctags.slice(initNum)
                                 .map(ctag => createCtagElm(ctag))//残りを作成    
                        )
                        list.find('[data-toggle="tooltip"]').tooltip();
                    });
                }
            );
        })();
    });
}

//カスタムタグの編集
function enableEditCtags(selector, dynamicSummaryTable, updatePath){
    const modal = $('#connectCtagsModal');
    const searchPath = $('#sidfm_internal_api_search_ctags_path').val();
    const selectedCtagsList = $('#connect-ctags-selected-ctags-list');
    const selectedCtagsListIds = $('#connect-ctags-selected-ctags-ids');
    const searchedCtagsListArea = $('#connect-ctags-searched-list-area');
    const searchedCtagsList = $('#connect-ctags-searched-list');
    const input = $('#connect-ctags-input');
    const inputTooltip = $('#connect-ctags-input-tooltip');
    const SearchedCtagsInfo = { fetching: false };
    inputTooltip.attr('data-original-title', '該当するカスタムタグはありません');

    //選択するとき
    const bindSelectEvent = (targetElm = null) => {
        (targetElm ? targetElm : searchedCtagsList.find('[data-select="ctag"]')).on('click', function(){
            const tag = $(this);
            const id = tag.data('id');
            const selectedData = JSON.parse(selectedCtagsListIds.val());
            if( selectedData.find(cId => cId == id) ){
                alert('選択済みのカスタムタグです。');
                return;
            }
            selectedData.push(id);
            selectedCtagsListIds.val(JSON.stringify(selectedData));
            const cpTag = tag.clone();
            //tag.remove();
            selectedCtagsList.append(cpTag);
            bindUnselectEvent(cpTag);
            //if( !searchedCtagsList.find('[data-select="ctag"]').length ) searchedCtagsListArea.hide();
        });
    };

    //選択を外すとき
    const bindUnselectEvent = (targetElm = null) => {
        (targetElm ? targetElm : selectedCtagsList.find('[data-select="ctag"]')).on('click', function(){
            const tag = $(this);
            const id = tag.data('id');
            const selectedData = JSON.parse(selectedCtagsListIds.val());
            selectedCtagsListIds.val(JSON.stringify(selectedData.filter(ctagId => ctagId != id)));
            //const cpTag = tag.clone();
            tag.tooltip('hide');
            tag.remove();
            /*const searchingName = input.val().toLowerCase();
            const tagName = cpTag.attr('data-name').toLowerCase();
            if( searchingName && tagName.indexOf(searchingName) !== -1 ){
                searchedCtagsList.append(cpTag);
                bindSelectEvent(cpTag);
                searchedCtagsListArea.show();
            }*/
        });
        (targetElm ? targetElm : selectedCtagsList.find('[data-select="ctag"]')).tooltip();
    };

    const createTagElm = (ctag) => {
        const styleProps = [];
        styleProps.push('background-color: ' + (ctag.bgcolor ? ctag.bgcolor : '#fff') + ';');
        if( ctag.color ){
            styleProps.push('color: ' + ctag.color + ';');
        }
        return(
            $('<span>').attr({
                class: 'searched-ctag ml-2 mr-2 mb-1 mt-1',
                'data-id': ctag.id,
                'data-select': 'ctag',
                'data-name': ctag.name,
                'data-toggle': 'tooltip',
                'data-title': 'カテゴリ: ' + escape_html_special_chars(ctag.category_name ? ctag.category_name : '全体'),
                'data-html': 'true',
                'data-placement': 'right',
                'data-boundary': 'window',
                'style': styleProps.join(' ')
            }).text(ctag.name)
        );
    };

    $(selector).on('click', function(){ //ボタン押下でモーダル開く
        const trigger = $(this);
        const targetId = $(this).data('id');
        const htmlid = $(this).data('htmlid');
        const sidfmdHostId = $(this).attr('data-hostid');
        if( sidfmdHostId ){
            $('#connect-ctags-host-id').val(sidfmdHostId);
        }
        $('#connect-ctags-target-id').val(htmlid);
        modal.find('.modal-title').text('カスタムタグの編集: ' + trigger.attr('data-title'));
        input.val('');
        searchedCtagsListArea.hide();
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction();
            if( roleOrCategoryError ) return;
            const paramsData = { id: htmlid };
            if( sidfmdHostId ){
                paramsData.sidfmd_host_id = sidfmdHostId;
            }
            $.post($('#sidfm_internal_api_get_ctags_by_htmlid_path').val(), paramsData, function(ctags){
                searchedCtagsList.html('');
                selectedCtagsList.html('');
                const ctagElms = [];
                const ctagIds = [];
                ctags.forEach( ctag => {
                    ctagElms.push(createTagElm(ctag));
                    ctagIds.push(ctag.id);
                });
                if( ctagElms.length ){
                    selectedCtagsList.append(ctagElms);
                    bindUnselectEvent();
                }
                selectedCtagsListIds.val(JSON.stringify(ctagIds));
            });
            modal.modal('show');
        })();
    });

    input.unbind('input');
    input.on('input', function(){ //タグの検索
        const keyword = $(this).val();
        if( SearchedCtagsInfo.fetching ){
            SearchedCtagsInfo.fetching.abort();
        }
        const paramsData = { keyword };//, except_ctag_ids: selectedCtagsListIds.val() };
        /*if( sidfmdHostId ){
            paramsData.sidfmd_host_id = sidfmdHostId;
        }*/
        SearchedCtagsInfo.fetching = $.post(
            searchPath,
            paramsData,
            function(ctags){
                const ctagElms = ctags.map( ctag => createTagElm(ctag) );
                searchedCtagsList.find('[data-toggle="tooltip"]').tooltip('hide');
                searchedCtagsList.html(''); 
                if( ctagElms.length ){
                    searchedCtagsListArea.show();
                    inputTooltip.tooltip('hide');
                    searchedCtagsList.append(ctagElms);
                    bindSelectEvent();
                    searchedCtagsList.find('[data-toggle="tooltip"]').tooltip();
                }else{
                    searchedCtagsListArea.hide();
                    inputTooltip.tooltip(keyword.length ? 'show' : 'hide');
                }
            }
        );
    });

    //実行
    const ctagConf = { conf: null };
    $('#exec-connect-ctags').unbind('click');
    $('#exec-connect-ctags').on('click', function(){
        (async () => {
            ctagConf.conf = confirm('カスタムタグを更新します。よろしいですか?');
            if( !ctagConf.conf ) return;
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                additionalAlertMessage: '更新に失敗しました。'
            });
            const paramsData = {
                html_id: $('#connect-ctags-target-id').val(),
                ctag_ids: $('#connect-ctags-selected-ctags-ids').val()
            };
            const sidfmdHostId = $('#connect-ctags-host-id').val();
            if( sidfmdHostId ) paramsData.sidfmd_host_id = sidfmdHostId;
            dynamicSummaryTable.executeProcess({
                paramsData,
                path: updatePath,
                additionalAlertMessage: '更新に失敗しました。'
            });
        })();
    });
}

//保存フィルタ
function enableStoreFilters({dynamicSummaryTable, summaryTableFilter, opts = {}}){
    const registPath = $('#sidfm_internal_api_regist_filter_path').val();
    const updatePath = $('#sidfm_internal_api_update_filter_path').val();
    const deletePath = $('#sidfm_internal_api_delete_filter_path').val();
    const getIsSameNamePath = $('#sidfm_internal_api_get_is_same_filter_name_path').val();
    const modal = $('#StoreFilterModal');
    const defaultSelectedType = 'private';
    const selectedFilterElm = $('#selected-store-filter-data');
    const selectedFilterTypeElm = $('#selected-store-filter-type');
    const selectPrivateElm = modal.find('[data-filtertype="' + defaultSelectedType + '"]');
    const selectGroupElm = modal.find('[data-filtertype="group"]');
    const selectedClass = 'selected';
    const selectCategoryElm = $('#select-store-filters-category');
    const selectCategoryArea = $('#select-store-filters-category-area');
    const inputNameElm = $('#input-for-store-filters-name');
    const execButton = $('#exec-store-filter');
    const storedFiltersListMenu = $('#stored-filters-list-menu');
    const actionTargetElm = $('#store-filter-action-target');
    const showStoredFiltersTriggerElm = $('#show-stored-filters');
    const isGroupFilterSelectable = $('#is-group-filter-selectable').val() === 'true';
    const isCategoryAllGroupFilterSelectable = $('#is-category-all-group-filter-selectable').val() === 'true';
    let isSwitchFilterTypeAreaAvailable = true;   

    const getIsSameName = ({name, type, categoryId = undefined, id}) => {
        const paramData = {
            name,
            type,
            keyword: dynamicSummaryTable.pageName,
        };
        if( categoryId !== undefined){
            paramData.category_id = categoryId;
        }
        if( id ){
            paramData.id = id;
        }
        return new Promise((resolve, reject) => {
            $.ajax({ 
                url: getIsSameNamePath,
                type: 'POST',
                data: paramData
            }).done(function(ret){
                resolve(ret === true);
            }).fail(function(){
                reloadOnFail('エラーが発生しました。画面をリロードします。');
                reject(false);
            });
        });
    };

    //保存/変更実行ボタンの活性/非活性
    const toggleExecButton = () => {
        if( selectedFilterTypeElm.val() === 'group' ){
            execButton.prop('disabled', (!inputNameElm.val() || selectCategoryElm.prop('disabled')));
        }else{
            execButton.prop('disabled', !inputNameElm.val());
        }
    };

    const setFilterTypeAreaMessage = (message) => {
        $('#switch-filter-type-area').attr('data-original-title', message).tooltip('show');
    };

    //フィルタ更新時に要素を更新後のものと差し替えるため、要素にidを付与
    //その時のidを取得
    const getElmId = (data) => {
        return data.type + data.id;
    };

    //保存フィルタ一覧の要素
    const createFiltersListMenuItem = (data) => {
        const itemElm = $('<div>').attr({
            class: 'dropdown-item g-brd-bottom g-brd-style-dotted g-brd-ddd d-flex select-filter align-items-center pr-2 pl-2',
            'data-filtersdata': JSON.stringify(data.filters),
            /*'data-toggle': 'tooltip',
            'data-title': data.name,
            'data-placement': 'left',
            'data-bundary': 'window',
            'data-flip': false,*/ //tooltipを出すと編集ボタンに被って押せなくなる
            'data-id': getElmId(data),
        }).append(
            $('<div>').attr('class', 'name-area mr-1').append(
                $('<i>').attr('class', (data.type === 'group' ? 'fas fa-user-friends ' : '') + 'mr-1'),
                $('<span>').text(data.name)
            )
        );
        let isEditable = false;
        if( data.type == 'group'){
            if( isGroupFilterSelectable && (data.sidfmd_category_id || isCategoryAllGroupFilterSelectable)){
                isEditable = true;
            }
        }else{
            isEditable = true;
        }
        if(isEditable){
            delete data['filters'];
            itemElm.append(
                $('<span>').attr('class', 'ml-auto edit-area').append(
                    $('<i>').attr('class', 'fas fa-pen-square color-edit update-filters').append(
                        $('<input>').attr({type: 'hidden', 'data-updatedata': true}).val(JSON.stringify(data))
                    ),
                    $('<i>').attr('class', 'ml-1 fas fa-minus-circle color-delete delete-filters').append(
                        $('<input>').attr({type: 'hidden', 'data-deletedata': true}).val(JSON.stringify(data))
                    )
                )
            );
        }
        return itemElm;
    };

    //イベントをbind
    const bindEvents = (filtersListMenuItem) => {
       //編集
       filtersListMenuItem.find('.update-filters').on('click', function(e){
           e.stopPropagation();
           const data = JSON.parse(filtersListMenuItem.find('[data-updatedata]').val());
           $('#store-filter-description').text('このフィルタの内容を、現在のフィルタの内容に変更します。');
           modal.find('.modal-title').text('フィルタ変更');
           execButton.text('更新');
           actionTargetElm.val('update');
           selectedFilterElm.val(JSON.stringify(data));
           selectedFilterTypeElm.val(data.type);
           if( data.type == 'private' ){
               selectCategoryArea.hide();
               selectPrivateElm.addClass(selectedClass);
               selectGroupElm.removeClass(selectedClass);
               selectCategoryElm.find('option:first').prop('selected', true);
           }else{
               selectCategoryArea.show();
               selectPrivateElm.removeClass(selectedClass);
               selectGroupElm.addClass(selectedClass);
               selectCategoryElm.val(data.sidfmd_category_id || 0);
           }
           //各項目の変更は不可。フィルタのみ変更
           selectCategoryElm.prop('disabled', true);
           inputNameElm.val(data.name).prop('disabled', true);
           execButton.prop('disabled', false);
           isSwitchFilterTypeAreaAvailable = false;
           setFilterTypeAreaMessage('この項目の変更はできません');
           storedFiltersListMenu.parent().dropdown('hide'); 
           modal.modal('show');
       });
       //削除
       filtersListMenuItem.find('.delete-filters').on('click', function(e){
           e.stopPropagation();
           const data = JSON.parse(filtersListMenuItem.find('[data-deletedata]').val());
           let conf = confirm(`「${data.name}」を削除してもよろしいですか?`);
           if( conf ){
               (async () => {
                   const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction();
                   if( roleOrCategoryError ) return;
                   $.post(deletePath, {type: data.type, id: data.id}, function(ret){
                       if( ret ){
                           const target = storedFiltersListMenu.find('[data-id="' + getElmId(data) + '"]');
                           //target.tooltip('hide');
                           target.remove();
                           if( !storedFiltersListMenu.find('[data-id]').length ){
                               showStoredFiltersTriggerElm.prop('disabled', true);
                           }
                       }else{
                           reloadOnFail('エラーが発生しました。画面をリロードします。');
                           return false;
                       }
                   }).fail(function(){
                       reloadOnFail('エラーが発生しました。画面をリロードします。');
                       return false;
                   });
               })();
           }
       });
       //filtersListMenuItem.tooltip();
       //フィルタ適用
       filtersListMenuItem.on('click', function(e){
           storedFiltersListMenu.find('.select-filter').removeClass('selected');
           $(this).addClass('selected');
           const filters = JSON.parse($(this).attr('data-filtersdata'));
           (async () => {
               const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction();
               if( roleOrCategoryError ) return;
               summaryTableFilter.executeBulkFilter(filters, { userHostOsnames: opts['userHostOsnames'] });
           })();
       });
    }

    //最初に登録フィルタを生成
    var storedFilters = JSON.parse($('#stored-filters').val());
    storedFilters.forEach(item => {
        const elm = createFiltersListMenuItem(item);
        storedFiltersListMenu.append(elm);
        bindEvents(elm);
    });
    if( !storedFilters.length ){
        showStoredFiltersTriggerElm.prop('disabled', true);
    }
    storedFilters = null; //以降使わない

    //新規追加
    $('#create-store-filters').on('click', function(){
        $('#store-filter-description').text('現在のフィルタ(サマリー表直上の検索ボタン)の状態を保存します。');
        modal.find('.modal-title').text('フィルタ追加');
        execButton.text('保存');
        inputNameElm.prop('disabled', false);
        actionTargetElm.val('create');
        selectedFilterTypeElm.val(defaultSelectedType);
        selectedFilterElm.val('{}');
        selectPrivateElm.addClass(selectedClass);
        selectGroupElm.removeClass(selectedClass);
        inputNameElm.val('');
        selectCategoryElm.find('option:first').prop('selected', true);
        selectCategoryArea.hide();
        toggleExecButton();
        isSwitchFilterTypeAreaAvailable = true;
        setFilterTypeAreaMessage(!isGroupFilterSelectable ? '個人フィルタのみ選択可能です' : '');
        modal.modal('show');
    });

    //個人フィルタ/グループフィルタの切り替え
    modal.find('[data-filtertype]').on('click', function(){
        if( !isGroupFilterSelectable || !isSwitchFilterTypeAreaAvailable ) return;
        const type = $(this).attr('data-filtertype');
        modal.find('[data-filtertype]').removeClass(selectedClass);
        $(this).addClass(selectedClass);
        selectedFilterTypeElm.val(type);
        if( type !== 'group') {
            selectCategoryArea.hide();
        }else{
            selectCategoryArea.show();
            selectCategoryElm.prop('disabled', false);
        }
        toggleExecButton();
    });

    //フィルタ名入力時
    inputNameElm.on('input', function(){
        toggleExecButton();
    });

    //保存/変更実行
    execButton.on('click', function(){
        const button = $(this);
        button.addClass('pointer-events-none');
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction();
            if( roleOrCategoryError ) return;
            const action = actionTargetElm.val();
            const paramData = { filters: $('#pages_filters').val(), keyword: dynamicSummaryTable.pageName };

            let isSameName = false;
            if( action === 'update' ){
                const filtersData = JSON.parse(selectedFilterElm.val());
                paramData.id = filtersData.id;
                paramData.type = filtersData.type;
                const isSameNameData = {
                    name: filtersData.name,
                    type: paramData.type,
                    id: paramData.id,
                };
                if( paramData.type === 'group' ){
                    paramData.category_id = selectCategoryElm.val();
                    isSameNameData.categoryId = paramData.category_id; 
                }
                isSameName = await getIsSameName(isSameNameData);
            }else{
                //新規
                paramData.type = selectedFilterTypeElm.val();
                paramData.name = inputNameElm.val();
                const isSameNameData = { type, name } = paramData;
                if( paramData.type === 'group' ){
                    paramData.category_id = selectCategoryElm.val();
                    isSameNameData.categoryId = paramData.category_id;
                }
                isSameName = await getIsSameName(paramData);
            }
            if( isSameName === true ){
                let conf = confirm(
                    (paramData.type === 'group' ? '指定したカテゴリに同じフィルタ名が登録されています。' : '同じフィルタ名が登録されています。') +
                    '\nこのまま登録しますか?'
                );
                if( !conf ) {
                    button.removeClass('pointer-events-none');
                    return
                }
            }
            $.post((action === 'update' ? updatePath : registPath), paramData, function(data){
                if( data ){
                    const elm = createFiltersListMenuItem(data);
                    if( action === 'update' ){
                        storedFiltersListMenu.find('[data-id="' + getElmId(data) + '"]')
                            .replaceWith(elm);
                    }else{
                        storedFiltersListMenu.append(elm);
                        showStoredFiltersTriggerElm.prop('disabled', false);
                    }
                    bindEvents(elm);
                }else{
                    reloadOnFail('エラーが発生しました。画面をリロードします。');
                    return false;
                }
            }).fail(function(){
                reloadOnFail('エラーが発生しました。画面をリロードします。');
                return false;
            });
            modal.modal('hide');
            button.removeClass('pointer-events-none');
        })();
    });
};

