#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

require 'net/http'

module SIDfm

  class VMConfig

    private

    def initialize(path = '/vm.config')
      @path = path
      @config = {}
      read_config
    end

    def read_config
      File.foreach(@path) do |line|
        case line
        when /\A#/ then
          # nop
        when /\A([0-9a-zA-Z_]+):\s+(.*)\Z/m then
          s = $1.to_sym
          @config[s] = $2.chomp
          if /\A"(.*)"\Z/ =~ @config[s] then
            @config[s] = $1
          elsif /\A'(.*)'\Z/ =~ @config[s] then
            @config[s] = $1
          end
        end
      end
    end

    public

    def get(key, default_value = nil)
      @config.has_key?(key) ? @config[key] : default_value
    end

    def sidapi_url
      get(:sidapi_url)
    end

    def lmapi_url
      get(:lmapi_url)
    end

    def get_proxy(https = false)
      if proxy_enable? then
        args = if https then
                 args = proxy_same_http_https? ? proxy_http : proxy_https
               else
                 args = proxy_http
               end
        Net::HTTP::Proxy(args[:host], args[:port], args[:user], args[:pass])
      else
        Net::HTTP::Proxy(nil)
      end
    end

    def proxy_enable?
      get(:proxy_enable, 'false') == 'true'
    end

    def proxy_same_http_https?
      get(:proxy_same_http_https, 'false') == 'true'
    end

    def proxy_http
      { :host => get(:proxy_http_host),
        :port => get(:proxy_http_port),
        :user => get(:proxy_http_user),
        :pass => get(:proxy_http_pass), }
    end

    def proxy_https
      { :host => get(:proxy_https_host),
        :port => get(:proxy_https_port),
        :user => get(:proxy_https_user),
        :pass => get(:proxy_https_pass), }
    end

  end

end

if $0 == __FILE__ then
  require 'pp'

  o = SIDfm::VMConfig.new

  pp o.get(:sidapi_url)
end
