#!/usr/bin/env ruby
# -*- coding: utf-8 -*-
#ルールのテストデータ作成用module
#必要なルールのデータだけ生成してテストする。(必要なデータのメソッドを呼び出す)

module SidfmdRuleData

  def create_rule(name, category_id: nil, position: nil, flg_enable: true)
    rule = SidfmdRule.create(
      sidfmd_category_id: category_id,
      name: name,
      position: position || 0,
      flg_enable: flg_enable
    )
  end

  # タイミング
  # 毎日実行
  def create_tp_hour_trigger(rule, hour: 1)
    trigger = SidfmdRuleTrigger.create(sidfmd_rule_id: rule.id)
    tp_hour_trigger = SidfmdRuleTpHour.create(
      sidfmd_rule_trigger_id: trigger.id,
      hour: hour
    )
  end

  # 定期実行
  def create_tp_interval_trigger(rule, value: 1, unit: 'hour')
    trigger = SidfmdRuleTrigger.create(sidfmd_rule_id: rule.id)
    tp_interval_trigger = SidfmdRuleTpInterval.create(
      sidfmd_rule_trigger_id: trigger.id,
      value: value,
      unit: unit
    )
  end

  # 脆弱性同期時
  def create_vulsync_trigger(rule, sidfmd_host_id: nil, sidfmd_category_id: nil)
    trigger = SidfmdRuleTrigger.create(sidfmd_rule_id: rule.id)
    vulsync_trigger = SidfmdRuleTVulsync.create(
      sidfmd_rule_trigger_id: trigger.id,
      sidfmd_host_id: sidfmd_host_id,
      sidfmd_category_id: sidfmd_category_id
    )
  end
  
  # 条件
  # プロダクト限定・除外の登録
  def create_cm_lapp_cond(rule, lproduct_ids: [], lversion_ids: [], is_match: true)
    created_lapp_subs = []
    if lproduct_ids.any? or lversion_ids.any? then
      cond = SidfmdRuleCond.create(sidfmd_rule_id: rule.id)
      app_match = SidfmdRuleCMatchApp.create(
        sidfmd_rule_cond_id: cond.id,
        inv: !is_match
      )
      lproduct_ids.each do |p_id|
        created_lapp_subs << SidfmdRuleCmLappSub.create(
          sidfmd_rule_c_match_app_id: app_match.id,
          sidfmd_lproduct_id: p_id
        )
      end
      lpid_num = 0
      lversion_ids.each do |v_id|
        created_lapp_subs  << SidfmdRuleCmLappSub.create(
          sidfmd_rule_c_match_app_id: app_match.id,
          sidfmd_lproduct_id: SidfmdLversion.find(v_id).sidfmd_lproduct_id,
          sidfmd_lversion_id: v_id
        )
      end
    end
    created_lapp_subs
  end

  # カテゴリ限定・除外の条件
  def create_cm_category_cond(rule, category_ids, is_match: true)
    cond = SidfmdRuleCond.create(sidfmd_rule_id: rule.id)
    c_match = SidfmdRuleCMatchCategory.create(
      sidfmd_rule_cond_id: cond.id,
      inv: !is_match
    )
    category_ids.each do |cat_id|
      SidfmdRuleCmCategorySub.create(
        sidfmd_rule_c_match_category_id: c_match.id,
        sidfmd_category_id: cat_id
      )
    end
  end

  # 操作
  # syslog送信
  def create_syslog_action(
    rule,
    action_target: SidfmdRuleAction::TARGET_VULS,
    action_proc_empty: true,
    protocol: 'udp',
    target: SidfmdRuleASendSyslog::IP_ADDRESS_IDX + '172.17.0.1',
    port: 514,
    hostname: 'sidfmvmsrv',
    ident: 'SIDfmVM',
    facility: 'LOCAL3',
    severity: 'INFO',
    message: 'test')

    action = SidfmdRuleAction.create(
      sidfmd_rule_id: rule.id,
      target: action_target,
      proc_empty: action_proc_empty
    )

    syslog_action = SidfmdRuleASendSyslog.create(
      sidfmd_rule_action_id: action.id,
      protocol: protocol,
      target: target,
      port: port,
      hostname: hostname,
      ident: ident,
      facility: facility,
      severity: severity,
      message: message
    )
  end

end
