require File.expand_path('../../../test_helper', __FILE__)
require_relative('../../test_data/sidfmd_rule_data')
require_relative('../../test_data/sidfmd_category_data')
require_relative('../../test_data/sidfmd_lproduct_data')

#sidfmd_rule.rb#lproducts_related_rule_infoメソッドのテスト
class SidfmdRuleLproductTest < ActiveSupport::TestCase
  include SidfmdRuleData, SidfmdCategoryData, SidfmdLproductData

  setup do
    lproduct_names = []
    10.times do |num|
      lproduct_names << "test#{num}"
    end
    lproduct_data = create_lproduct_data(lproduct_names, [ 'ver1', 'ver2', 'ver3' ])
    @lproducts = lproduct_data[:lproducts]
    @lversions = lproduct_data[:lversions]
  end

  def create_lproduct_data(lproduct_names, version_names)
    lproducts = lproduct_names.map{|name| create_lproduct(name)}
    lversions = []
    lversion_data = version_names.map.with_index{|ver, idx|
      { :version => ver, :priority => idx + 1 }
    }
    lproducts.each do |lp|
      lversions += create_lversions(lp, lversion_data)
    end
    { :lproducts => lproducts, :lversions => lversions }
  end

  def create_category_data
    @categories = [ 'test1', 'test2', 'test3' ].map{|cat_name|
      create_category(cat_name)
    }
  end

  test "lproducts_related_rule_infoメソッド:ローカルプロダクト削除(管理者)" do
    puts('start lproducts_related_rule_infoメソッド:ローカルプロダクト削除(管理者)')
    rule_names = [
      'test',
      'テスト',
      '動作確認',
      'test2'
    ]
    rules = rule_names.map{|rule_name| rule = create_rule(rule_name)}
    cm_lapp_subs = []
    rules.each_with_index do |rule, idx|
      case idx #各タイミングを作成(プロダクトの条件には関係ない)
      when 0 then
        create_tp_hour_trigger rule
      when 1 then
        create_tp_interval_trigger rule
      else
        create_vulsync_trigger rule
      end
      cm_lapp_subs += create_cm_lapp_cond(
        rule,
        lproduct_ids: @lproducts.map{|lp| lp.id},
        lversion_ids: @lversions.map{|lv| lv.id},
        is_match: idx / 2 == 0
      )
      create_syslog_action(rule)
    end
    param_data = {
      :role_admin => true,
      :user_category_ids => nil,
      :action => 'delete',
      :messages_necessary => true,
    }
    param_data[:delete_lproduct_ids] = @lproducts.map{|lp| lp.id}
    param_data[:delete_lversion_ids] = []
    #条件がなくなるルールが削除対象になること
    rule_info = SidfmdRule.lproducts_related_rule_info(param_data)
    assert rule_info[:delete_rule_ids].size == rules.size
    assert rule_info[:delete_cond_ids].size == 0
    assert rule_info[:delete_action_ids].size == 0
    assert rule_info[:delete_cm_lapp_ids].size == 0

    #プロダクト限定・除外からプロダクトが削除対象になること
    param_data[:delete_lproduct_ids] = @lproducts.map{|lp| lp.id}.slice(0..1)
    rule_info = SidfmdRule.lproducts_related_rule_info(param_data)
    assert rule_info[:delete_rule_ids].size == 0
    assert rule_info[:delete_cond_ids].size == 0
    assert rule_info[:delete_action_ids].size == 0
    assert rule_info[:delete_cm_lapp_ids].size == cm_lapp_subs.select{|lapp_sub|
      param_data[:delete_lproduct_ids].include?(lapp_sub.sidfmd_lproduct_id)
    }.size

    #プロダクト限定・除外が削除されても条件が１つ以上残るなら、プロダクト限定・除外条件だけが削除対象になること
    create_category_data
    rules.each do |rule|
      create_cm_category_cond(rule, @categories.map(&:id))
    end
    param_data[:delete_lproduct_ids] = @lproducts.map{|lp| lp.id}
    rule_info = SidfmdRule.lproducts_related_rule_info(param_data)
    assert rule_info[:delete_rule_ids].size == 0
    assert rule_info[:delete_cond_ids].size == cm_lapp_subs.map{|lapp_sub|
      lapp_sub.sidfmd_rule_c_match_app.sidfmd_rule_cond.id
    }.uniq.size
    assert rule_info[:delete_action_ids].size == 0
    assert rule_info[:delete_cm_lapp_ids].size == 0
    puts('end') 
  end

  test "lproducts_related_rule_infoメソッド:ローカルバージョン削除(管理者)" do
    puts('start lproducts_related_rule_infoメソッド:ローカルバージョン削除(管理者)')
    rule_names = [
      'test',
      'テスト',
      '動作確認',
      'test2'
    ]
    rules = rule_names.map{|rule_name| rule = create_rule(rule_name)}
    cm_lapp_subs = []
    rules.each_with_index do |rule, idx|
      create_tp_hour_trigger(rule)
      cm_lapp_subs += create_cm_lapp_cond(
        rule,
        lversion_ids: @lversions.map{|lv| lv.id},
        is_match: idx / 2 == 0
      )
      create_syslog_action(rule)
    end
    param_data = {
      :role_admin => true,
      :user_category_ids => nil,
      :action => 'delete',
      :messages_necessary => true,
    }
    param_data[:delete_lproduct_ids] = []
    param_data[:delete_lversion_ids] = @lversions.map{|lv| lv.id}
    #条件がなくなるルールが削除対象になること
    rule_info = SidfmdRule.lproducts_related_rule_info(param_data)
    assert rule_info[:delete_rule_ids].size == rules.size
    assert rule_info[:delete_cond_ids].size == 0
    assert rule_info[:delete_action_ids].size == 0
    assert rule_info[:delete_cm_lapp_ids].size == 0
 
    #プロダクト限定・除外からバージョンが削除対象になること
    param_data[:delete_lversion_ids] = @lversions.map{|lv| lv.id}.uniq.slice(0..3)
    rule_info = SidfmdRule.lproducts_related_rule_info(param_data)
    assert rule_info[:delete_rule_ids].size == 0
    assert rule_info[:delete_cond_ids].size == 0
    assert rule_info[:delete_action_ids].size == 0
    assert rule_info[:delete_cm_lapp_ids].size == cm_lapp_subs.select{|lapp_sub|
      param_data[:delete_lversion_ids].include?(lapp_sub.sidfmd_lversion_id)
    }.size

    #プロダクト限定・除外が削除されても条件が１つ以上残るなら、プロダクト限定・除外条件だけが削除対象になること
    create_category_data
    rules.each do |rule|
      create_cm_category_cond(rule, @categories.map(&:id))
    end
    param_data[:delete_lversion_ids] = @lversions.map{|lv| lv.id}
    rule_info = SidfmdRule.lproducts_related_rule_info(param_data)
    assert rule_info[:delete_rule_ids].size == 0
    assert rule_info[:delete_cond_ids].size == cm_lapp_subs.map{|lapp_sub|
      lapp_sub.sidfmd_rule_c_match_app.sidfmd_rule_cond.id
    }.uniq.size
    assert rule_info[:delete_action_ids].size == 0
    assert rule_info[:delete_cm_lapp_ids].size == 0
    puts('end')
  end

  test "lproducts_related_rule_infoメソッド:ローカルプロダクトとローカルバージョン両方同時に削除(管理者)" do
    puts('start lproducts_related_rule_infoメソッド:ローカルプロダクトとローカルバージョン両方同時に削除(管理者)')
    rule_names = [
      'test',
      'テスト',
      '動作確認'
    ]
    rules = rule_names.map{|rule_name| rule = create_rule(rule_name)}
    cm_lapp_subs = []
    lpids1 = @lproducts.map{|lp| lp.id}.slice(0..1)
    lpids2 = [ @lproducts[2].id ]
    lpids3 = [ @lproducts[3].id ]
    lvids1 = @lversions.select{|lv| @lproducts[4].id == lv.sidfmd_lproduct_id }.map{|lv| lv.id}
    lvids2 = @lversions.select{|lv| @lproducts[5].id == lv.sidfmd_lproduct_id }.map{|lv| lv.id}
    lvids3 = @lversions.select{|lv| @lproducts[6].id == lv.sidfmd_lproduct_id }.map{|lv| lv.id}
    rule_1_lvids = []
    rules.each_with_index do |rule, idx|
      case idx
      when 0 then
        create_tp_hour_trigger rule #タイミングを作成(プロダクトの条件には関係ない)
        cm_lapp_subs += create_cm_lapp_cond( #プロダクト限定・除外の両方を設定してテスト
          rule,
          lproduct_ids: lpids1,
          lversion_ids: lvids1,
          is_match: true
        )
        cm_lapp_subs += create_cm_lapp_cond(
          rule,
          lproduct_ids: lpids2,
          lversion_ids: lvids2,
          is_match: false
        )
      when 1 then
        create_vulsync_trigger rule
        cm_lapp_subs += create_cm_lapp_cond( #プロダクト限定・除外の両方を設定してテスト
          rule,
          lproduct_ids: lpids1,
          lversion_ids: lvids1,
          is_match: false
        )
        cm_lapp_subs += create_cm_lapp_cond(
          rule,
          lproduct_ids: lpids2,
          lversion_ids: lvids2,
          is_match: true
        )
      else
        cm_lapp_subs += create_cm_lapp_cond( #プロダクト限定で0と1とは異なるプロダクト・バージョン
          rule,
          lproduct_ids: lpids3,
          lversion_ids: lvids3,
          is_match: false
        )
      end
      create_syslog_action(rule)
    end
    param_data = {
      :role_admin => true,
      :user_category_ids => nil,
      :action => 'delete',
      :messages_necessary => true,
    }
    param_data[:delete_lproduct_ids] = lpids1 + lpids2 + [ lpids3[0] ]
    param_data[:delete_lversion_ids] = lvids1 + lvids2 + [ lvids3[0] ]
    #条件がなくなるルールが削除対象になること + プロダクト限定・除外からプロダクトが削除対象になること
    rule_info = SidfmdRule.lproducts_related_rule_info(param_data)
    assert rule_info[:delete_rule_ids].size == 2
    assert rule_info[:delete_cond_ids].size == 0
    assert rule_info[:delete_action_ids].size == 0
    assert rule_info[:delete_cm_lapp_ids].size == 2

    #プロダクト限定・除外が削除されても条件が１つ以上残るなら、プロダクト限定・除外条件だけが削除対象になること
    param_data[:delete_lproduct_ids] = lpids1
    param_data[:delete_lversion_ids] = lvids1
    rule_info = SidfmdRule.lproducts_related_rule_info(param_data)
    assert rule_info[:delete_rule_ids].size == 0
    assert rule_info[:delete_cond_ids].size == 2
    assert rule_info[:delete_action_ids].size == 0
    assert rule_info[:delete_cm_lapp_ids].size == 0
    puts('end')
  end
  
end
