# -*- coding: utf-8 -*-

class SidfmdRuleCond < ActiveRecord::Base
  belongs_to :sidfmd_rule

  has_one :sidfmd_rule_c_issue_update, dependent: :delete
  has_one :sidfmd_rule_c_sri_above, dependent: :delete
  has_one :sidfmd_rule_c_sri_below, dependent: :delete
  has_one :sidfmd_rule_c_cvss_above, dependent: :delete
  has_one :sidfmd_rule_c_cvss_below, dependent: :delete
  has_one :sidfmd_rule_c_cvss_notreg, dependent: :delete
  has_one :sidfmd_rule_c_warning, dependent: :delete
  has_one :sidfmd_rule_c_history_post, dependent: :destroy
  has_one :sidfmd_rule_c_duedate_expired, dependent: :delete
  has_one :sidfmd_rule_c_duedate_approaching, dependent: :delete
  has_one :sidfmd_rule_c_match_category, dependent: :destroy
  has_one :sidfmd_rule_c_match_host, dependent: :destroy
  has_one :sidfmd_rule_c_match_app, dependent: :destroy
  has_one :sidfmd_rule_c_status, dependent: :delete
  has_one :sidfmd_rule_c_cvss_exploitable, dependent: :delete
  has_one :sidfmd_rule_c_cvss_vector, dependent: :delete
  has_one :sidfmd_rule_c_host_syncing, dependent: :delete
  has_one :sidfmd_rule_c_timerange, dependent: :delete
  has_one :sidfmd_rule_c_duedate_undef, dependent: :delete
  has_one :sidfmd_rule_c_ctag, dependent: :delete

  # 関連づけられている条件を取得する。
  def detail
    case
    when sidfmd_rule_c_issue_update        then sidfmd_rule_c_issue_update
    when sidfmd_rule_c_sri_above           then sidfmd_rule_c_sri_above
    when sidfmd_rule_c_sri_below           then sidfmd_rule_c_sri_below
    when sidfmd_rule_c_cvss_above          then sidfmd_rule_c_cvss_above
    when sidfmd_rule_c_cvss_below          then sidfmd_rule_c_cvss_below
    when sidfmd_rule_c_cvss_notreg         then sidfmd_rule_c_cvss_notreg
    when sidfmd_rule_c_warning             then sidfmd_rule_c_warning
    when sidfmd_rule_c_history_post        then sidfmd_rule_c_history_post
    when sidfmd_rule_c_duedate_undef       then sidfmd_rule_c_duedate_undef
    when sidfmd_rule_c_duedate_expired     then sidfmd_rule_c_duedate_expired
    when sidfmd_rule_c_duedate_approaching then sidfmd_rule_c_duedate_approaching
    when sidfmd_rule_c_match_category      then sidfmd_rule_c_match_category
    when sidfmd_rule_c_match_host          then sidfmd_rule_c_match_host
    when sidfmd_rule_c_match_app           then sidfmd_rule_c_match_app
    when sidfmd_rule_c_status              then sidfmd_rule_c_status
    when sidfmd_rule_c_cvss_exploitable    then sidfmd_rule_c_cvss_exploitable
    when sidfmd_rule_c_cvss_vector         then sidfmd_rule_c_cvss_vector
    when sidfmd_rule_c_host_syncing        then sidfmd_rule_c_host_syncing
    when sidfmd_rule_c_timerange           then sidfmd_rule_c_timerange
    when sidfmd_rule_c_ctag                then sidfmd_rule_c_ctag
    else nil
    end
  end

  # ログを追加する
  def log(type, message)
    self.with_lock do
      self.update_attribute :logs, SidfmdRule.log_append(self.logs, type, SidfmdRule::LOG_COND, message)
    end
  end

  def log_info(message)
    log(SidfmdRule::LOG_INFO, message)
  end

  def log_warn(message)
    log(SidfmdRule::LOG_WARN, message)
  end

  def log_err(message)
    log(SidfmdRule::LOG_ERR, message)
  end

end
