#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

module SidfmHostControllerSupport
  # 既存のメソッドとメソッド名が被らないように注意
 
  DAY_SECONDS = 24 * 60 * 60
 
  # ソート
  def sort_host_content(content:, order:, host_id:, index:, corresp_required:)
    case order
    when 'warning_asc','warning_desc'then
      content = content.joins(:sidfmd_vv_scsids).where(:sidfmd_vv_scsids => {:sidfmd_host_id => host_id}).order(%|
        CASE WHEN status_id IN (SELECT unnest(ARRAY[#{SidfmdIssue::STATUS_UNFINISHED.join(', ')}])) THEN
          CASE WHEN #{index == :sri ? "sri >= #{corresp_required[:sri]}" : "cvss IS NULL OR cvss >= #{corresp_required[:cvss]}"} THEN
            0
          ELSE
            100
          END
        ELSE
          100
        END
        #{order == 'warning_asc' ? 'ASC' : 'DESC'}
      |).order(%| DATE_TRUNC('DAY', sidfmd_vv_scsids.created_on) #{order == 'warning_asc' ? 'ASC' : 'DESC'} |)
        .order(%| LOWER(title COLLATE "C") #{order == 'warning_asc' ? 'ASC' : 'DESC'} |)

    when 'status_asc', 'status_desc' then
      content = content.joins(:sidfmd_vv_scsids)
                                               .where(:sidfmd_vv_scsids => { :sidfmd_host_id => host_id })
                                               .order(%| sidfmd_vv_scsids.status_id #{order == 'status_asc' ? 'ASC' : 'DESC'} |)
                                               .order(%| DATE_TRUNC('DAY', sidfmd_vv_scsids.created_on) #{order == 'status_asc' ? 'ASC' : 'DESC'} |)
                                               .order(%| LOWER(title COLLATE "C") #{order == 'status_asc' ? 'ASC' : 'DESC'} |)
    when 'sri_asc', 'sri_desc' then
      content = content.joins(:sidfmd_vv_scsids)
                                               .where(:sidfmd_vv_scsids => { :sidfmd_host_id => host_id })
                                               .order(%| sri #{order == 'sri_asc' ? 'DESC' : 'ASC'} |)
                                               .order(%| DATE_TRUNC('DAY', sidfmd_vv_scsids.created_on) #{order == 'sri_asc' ? 'ASC' : 'DESC'} |)
                                               .order(%| LOWER(title COLLATE "C") #{order == 'sri_asc' ? 'ASC' : 'DESC'} |)
    when 'cvss_asc', 'cvss_desc' then
      content = content.joins(:sidfmd_vv_scsids)
                                               .where(:sidfmd_vv_scsids => { :sidfmd_host_id => host_id })
                                               .order(%| cvss #{order == 'cvss_asc' ? 'DESC NULLS LAST' : 'ASC NULLS FIRST'} |)
                                               .order(%| DATE_TRUNC('DAY', sidfmd_vv_scsids.created_on) #{order == 'cvss_asc' ? 'ASC' : 'DESC'} |)
                                               .order(%| LOWER(title COLLATE "C") #{order == 'cvss_asc' ? 'ASC' : 'DESC'} |)
    when 'area_asc', 'area_desc' then
      content = content.joins(:sidfmd_vv_scsids)
                                               .where(:sidfmd_vv_scsids => { :sidfmd_host_id => host_id })
                                               .order(%|
                                                 CASE
                                                 WHEN html_id > 0 THEN
                                                   softek_area_id
                                                 ELSE
                                                   -10
                                                END
                                                #{order == 'area_asc' ? 'DESC' : 'ASC'}
                                              |)
                                              .order(%| DATE_TRUNC('DAY', sidfmd_vv_scsids.created_on) #{order == 'area_asc' ? 'ASC' : 'DESC'} |)
                                              .order(%| LOWER(title COLLATE "C") #{order == 'area_asc' ? 'ASC' : 'DESC'} |)
    when 'exploitable_asc',  'exploitable_desc' then
      content = content.joins(:sidfmd_vv_scsids)
                                               .where(:sidfmd_vv_scsids => { :sidfmd_host_id => host_id })
                                               .order(%|
                                                 CASE
                                                 WHEN html_id > 0 THEN
                                                   CASE
                                                   WHEN sidfmd_vv_scsids.status_id NOT IN (SELECT unnest(ARRAY[#{SidfmdIssue::STATUS_UNFINISHED.join(', ')}])) THEN
                                                     1600 
                                                   WHEN sidfmd_vul_views.exploitable IS NULL OR sidfmd_vv_scsids.cvss IS NULL THEN
                                                     1500
                                                   WHEN sidfmd_vul_views.exploitable IS TRUE THEN
                                                     500
                                                   ELSE
                                                     1000
                                                   END
                                                 ELSE
                                                   1250
                                                 END
                                                 #{order == 'exploitable_asc' ? 'ASC' : 'DESC'}
                                               |)
                                               .order(%| DATE_TRUNC('DAY', sidfmd_vv_scsids.created_on) #{order == 'exploitable_asc' ? 'DESC' : 'ASC'} |)
                                               .order(%| LOWER(title COLLATE "C") #{order == 'exploitable_asc' ? 'ASC' : 'DESC'} |)
    when  'elapseddays_asc', 'elapseddays_desc' then
      content = content.joins(:sidfmd_vv_scsids)
                                               .where(:sidfmd_vv_scsids => { :sidfmd_host_id => host_id })
                                               .order(%| 
                                                 CASE 
                                                 WHEN status_id IN (SELECT unnest(ARRAY[#{SidfmdIssue::STATUS_UNFINISHED.join(', ')}])) THEN
                                                   sidfmd_vv_scsids.created_on
                                                 ELSE
                                                   NULL
                                                 END
                                                 #{order == 'elapseddays_asc' ? 'ASC NULLS LAST' : 'DESC NULLS FIRST'}
                                               |)
                                               .order(%| LOWER(title COLLATE "C") #{order == 'elapseddays_asc' ? 'ASC' : 'DESC'} |)    

    when 'title_asc', 'title_desc' then
      content = content.order(%| abs(html_id) #{order == 'title_asc' ? 'ASC' : 'DESC'} |)
                                               .order(%| LOWER(title COLLATE "C") #{order == 'title_asc' ? 'ASC' : 'DESC'} |)
    when 'journals_asc', 'journals_desc' then
      content = content.joins(:sidfmd_vv_scsids)
                                               .where(:sidfmd_vv_scsids => { :sidfmd_host_id => host_id })
                                               .joins(%|
        LEFT OUTER JOIN(
          SELECT 
            html_id, 
            COUNT(journals.id) AS count_journals
          FROM
            sidfmd_issues INNER JOIN issues ON sidfmd_issues.issue_id = issues.id
            LEFT OUTER JOIN journals ON journals.journalized_id = issues.id
          WHERE sidfmd_issues.sidfmd_host_id = #{host_id}
          GROUP BY html_id
        ) AS order_count_journals
        ON sidfmd_vul_views.html_id = order_count_journals.html_id
      |).order(%| count_journals #{order == 'journals_asc' ? 'DESC' : 'ASC'} |)
        .order(%| DATE_TRUNC('DAY', sidfmd_vv_scsids.created_on) #{order == 'journals_asc' ? 'DESC' : 'ASC'} |)
        .order(%| LOWER(title COLLATE "C") #{order == 'journals_asc' ? 'ASC' : 'DESC'} |)
    when 'duedate_asc', 'duedate_desc' then
      content = content.joins(:sidfmd_vv_scsids)
                                               .where(:sidfmd_vv_scsids => { :sidfmd_host_id => host_id })
                                               .order(%|
                                                 CASE
                                                 WHEN sidfmd_vv_scsids.due_date IS NULL THEN
                                                   1000
                                                 ELSE
                                                   500
                                                 END
                                                 #{order == 'duedate_asc' ? 'ASC' : 'DESC'}
                                               |)
                                               .order(%|
                                                 CASE
                                                 WHEN sidfmd_vv_scsids.due_date IS NULL THEN
                                                   CAST('1970-01-01' AS DATE)
                                                 ELSE
                                                   sidfmd_vv_scsids.due_date
                                                 END
                                                 #{order == 'duedate_asc' ? 'ASC' : 'DESC'}
                                               |)
                                               .order(%| DATE_TRUNC('DAY', sidfmd_vv_scsids.created_on) #{order == 'duedate_asc' ? 'ASC' : 'DESC'}|)
                                               .order(%| LOWER(title COLLATE "C") #{order == 'duedate_asc' ? 'ASC' : 'DESC'} |)
    end
    content
  end

  # フィルタの初期値
  def default_host_filters
    filters = {}
    filters[:warning] = { :values => [ 'warning', 'others', 'all' ].map(&:freeze).freeze, :default => 'all'.freeze }
    filters[:status] = {
      :values => (SidfmdIssue::STATUS_ALL.collect {|x| x.to_s } +
                  [ SidfmdIssue::STATUS_UNFINISHED.collect {|x| x.to_s }.join(':') ] +
                  [ SidfmdIssue::STATUS_FINISHED.collect {|x| x.to_s }.join(':') ] +
                  [ '0' ]).map(&:freeze).freeze, :default => '0'.freeze }
    filters[:sris] = {
      :values => SidfmdIssueSri::VALUES.map{|v| [v.to_s, v]}.to_h,
      :default => [].freeze
    }
    filters[:cvsses] = {
      :values => SidfmdIssueCve::DISPLAY_SORTED_LABELS.map{|v| [v.to_s, v]}.to_h,
      :default => [].freeze
    }
    filters[:area] = { :values => [ '3', '2', '1', '' ].map(&:freeze).freeze, :default => ''.freeze }
    filters[:exploitable] = { :values => [ 'true', '' ].map(&:freeze).freeze, :default => ''.freeze }
    filters[:journal] = { :values => nil, :default => { :period => '::', :filter_status_change => false, :filter_duedate_change => false } }
    filters[:due_date] = { :values => nil, :default => '::' }
    filters[:keyword] = { :values => nil, :default => '' }
    filters[:date] = {
      :values => nil,
      :default => {
        :detected_date => '::',
        :created_date => '::',
        :updated_date => '::',
      }
    }
    filters[:cvss_vector] = {
      :type_source => { #type_sourceはdefaultでは型が不明な場合に使う。各値には期待されるデータと同じ型のデータを入れる
        :cvss_v2 => SidfmdIssueCve.get_cvss_v2_vectors.map{|vector, value| [ vector.to_sym, value['values'].keys.first.to_s ] }.to_h,
        :cvss_v3 => SidfmdIssueCve.get_cvss_v3_vectors.map{|vector, value| [ vector.to_sym, value['values'].keys.first.to_s ] }.to_h,
        :is_target_unregistered => false,
      },
      :values => nil,
      :default => {
        :cvss_v2 => SidfmdIssueCve.get_cvss_v2_vectors.keys.map{|vector| [ vector.to_sym, nil ] }.to_h,
        :cvss_v3 => SidfmdIssueCve.get_cvss_v3_vectors.keys.map{|vector| [ vector.to_sym, nil ] }.to_h,
        :is_target_unregistered => false,
      }
    }
    filters[:products] = {
      :type_source => {
        :product_ids => [0],
        :version_ids => [0],
        :lproduct_ids => [0],
        :lversion_ids => [0]
      },
      :values => nil,
      :default => {
        :product_ids => [],
        :version_ids => [],
        :lproduct_ids => [],
        :lversion_ids => []
      }
    }
    filters[:ctags] = { :values => nil, :default => { :ids => [] } }
    filters
  end

  # フィルタ適用
  def filter_host_content(content:, filters:, role_admin:, user_category_ids:, user_host_ids:, host_id:, index:, corresp_required:)
    #プロダクト検索
    if filters[:products].values.select{|v| v.size > 0 }.any? then
      prod_info = filters[:products].symbolize_keys
      issue_html_ids = []
      if prod_info[:product_ids] and prod_info[:product_ids].any? then
        #productが検索対象
        issues = SidfmdIssue.where(
          "id IN (?) OR id IN (?)",
          SidfmdIssueAffprod.where(:product_id => prod_info[:product_ids]).select(:sidfmd_issue_id),
          SidfmdIssueAffver.where(:product_id => prod_info[:product_ids]).select(:sidfmd_issue_id)
        )
        unless role_admin then
          issues = issues.where(:sidfmd_host_id => user_host_ids)
        end
        issue_html_ids |= issues.pluck(:html_id)
      end
      if prod_info[:version_ids] and prod_info[:version_ids].any? then
        #versionが検索対象
        issues = SidfmdIssue.where(
          :id => SidfmdIssueAffver.where(:product_version_id => prod_info[:version_ids]).pluck(:sidfmd_issue_id)
        )
        unless role_admin then
          issues = issues.where(:sidfmd_host_id => user_host_ids)
        end
        issue_html_ids |= issues.pluck(:html_id)
      end
      if prod_info[:lproduct_ids] and prod_info[:lproduct_ids].any? then
        #lproductが検索対象
        vuls = SidfmdLocalVul.joins(:sidfmd_local_vul_lapps)
                             .joins('LEFT OUTER JOIN sidfmd_issues ON sidfmd_issues.html_id = sidfmd_local_vuls.html_id')
                             .where('sidfmd_local_vul_lapps.sidfmd_lproduct_id IN (?)', prod_info[:lproduct_ids])
                             .select('sidfmd_issues.html_id AS html_id')
        unless role_admin then
          vuls = vuls.where(:sidfmd_issues => { :sidfmd_host_id => user_host_ids })
        end
        issue_html_ids |= vuls.pluck(:html_id)
      end
      if prod_info[:lversion_ids] and prod_info[:lversion_ids].any? then
        #lversionが検索対象
        vuls = SidfmdLocalVul.joins(:sidfmd_local_vul_lapps)
                             .joins('LEFT OUTER JOIN sidfmd_issues ON sidfmd_issues.html_id = sidfmd_local_vuls.html_id')
                             .where('sidfmd_local_vul_lapps.sidfmd_lversion_id IN (?)', prod_info[:lversion_ids])
                             .select('sidfmd_issues.html_id AS html_id')
        unless role_admin then
          vuls = vuls.where(:sidfmd_issues => { :sidfmd_host_id => user_host_ids })
        end
        issue_html_ids |= vuls.pluck(:html_id)
      end

      if issue_html_ids.any? then
        content = content.where(:html_id => issue_html_ids)
      else
        content = content.where(:html_id => 0)
      end
    end

    #履歴
    if filters[:journal][:period] != '::' then
      if /^(in|rev):(\d{4}\/\d{2}\/\d{2}|):(\d{4}\/\d{2}\/\d{2}|)$/ =~ filters[:journal][:period] then
        orig_flg_rev = Regexp.last_match(1)
        orig_begin_date = Regexp.last_match(2)
        orig_end_date = Regexp.last_match(3)
        if orig_flg_rev.size > 0 and orig_begin_date.size == 0 and orig_end_date.size == 0 then
          filters[:journal][:period] = '::'
        else
          flg_rev = orig_flg_rev == 'rev'
          begin_date = orig_begin_date.size > 0 ? Date.strptime(orig_begin_date, '%Y/%m/%d') : nil
          end_date = orig_end_date.size > 0 ? Date.strptime(orig_end_date, '%Y/%m/%d') : nil
          begin_date = begin_date.to_time if begin_date
          end_date = end_date.to_time + DAY_SECONDS - 1 if end_date

          j_html_ids = Journal.joins(:issue)
                              .joins('INNER JOIN sidfmd_issues ON issues.id = sidfmd_issues.issue_id')
                              .joins('LEFT OUTER JOIN journal_details ON journal_details.journal_id = journals.id')
                              .where('sidfmd_issues.sidfmd_host_id = ?', host_id)
                              .select('sidfmd_issues.html_id')
          if flg_rev then
            if begin_date and end_date then
              j_html_ids = j_html_ids.where.not(:journals => { :created_on => begin_date..end_date })
            elsif begin_date then
              j_html_ids = j_html_ids.where.not('journals.created_on >= ?', begin_date)
            elsif end_date then
              j_html_ids = j_html_ids.where.not('journals.created_on <= ?', end_date)
            else
              # last resort
              filters[:journal][:period] = '::'
            end
          else
            if begin_date and end_date then
              j_html_ids = j_html_ids.where(:journals => { :created_on => begin_date..end_date })
            elsif begin_date then
              j_html_ids = j_html_ids.where('journals.created_on >= ?', begin_date)
            elsif end_date then
              j_html_ids = j_html_ids.where('journals.created_on <= ?', end_date)
            else
              # last resort
              filters[:journal][:period] = '::'
            end
          end
          if filters[:journal][:period] != '::' then
            if filters[:journal][:filter_status_change].to_s == 'true' and filters[:journal][:filter_duedate_change].to_s == 'true' then
              f_html_ids = j_html_ids.where(:journal_details => { :prop_key => 'status_id' }).pluck('sidfmd_issues.html_id') &
                           j_html_ids.where(:journal_details => { :prop_key => 'due_date' }).pluck('sidfmd_issues.html_id')
              if f_html_ids.any? then
                j_html_ids = j_html_ids.where(%|
                  sidfmd_issues.html_id IN (SELECT unnest(ARRAY[#{f_html_ids.join(', ')}]))
                |)
              else
                j_html_ids = j_html_ids.where(:journals => { :id => 0 })
              end
            elsif filters[:journal][:filter_status_change].to_s == 'true' then
              j_html_ids = j_html_ids.where(:journal_details => { :prop_key => 'status_id' })
            elsif filters[:journal][:filter_duedate_change].to_s == 'true' then
              j_html_ids = j_html_ids.where(:journal_details => { :prop_key => 'due_date' })
            end
            content = content.where(:html_id => j_html_ids)
          end
        end
      else
        filters[:journal][:period] = '::'
      end
    end

    # 要注意
    if filters[:warning] == 'warning' then
      content = content.where(
        :id => SidfmdVvScsid.where(:sidfmd_host_id => host_id)
                            .where(%| status_id IN (SELECT unnest(ARRAY[#{SidfmdIssue::STATUS_UNFINISHED.join(', ')}])) |)
                            .where(%| #{index == :sri ? "sri >= #{corresp_required[:sri]}" : "cvss IS NULL OR cvss >= #{corresp_required[:cvss]}"} |)
                            .select(:sidfmd_vul_view_id)
      )
    elsif filters[:warning] == 'others' then
      content = content.where(
        :id => SidfmdVvScsid.where(:sidfmd_host_id => host_id)
                            .where(%| status_id NOT IN (SELECT unnest(ARRAY[#{SidfmdIssue::STATUS_UNFINISHED.join(', ')}])) OR
                                      #{index == :sri ? "sri < #{corresp_required[:sri]}" : "(cvss IS NOT NULL AND cvss < #{corresp_required[:cvss]})"} 
                            |)
                            .select(:sidfmd_vul_view_id)
      )
    end
    # 対応状況
    if /\A(\d):(\d)\z/ =~ filters[:status] then
      status1 = Regexp.last_match(1).to_i
      status2 = Regexp.last_match(2).to_i
      content = content.where(
        :id => SidfmdVvScsid.where(:sidfmd_host_id => host_id).where(:status_id => [ status1, status2 ]).select(:sidfmd_vul_view_id)
      )
    elsif filters[:status] != '0' then
      content = content.where(
        :id => SidfmdVvScsid.where(:sidfmd_host_id => host_id).where(:status_id => filters[:status].to_i).select(:sidfmd_vul_view_id)
      )
    end
    # SRI
    if filters[:sris].any? then
      content = content.where(
        :id => SidfmdVvScsid.where(:sidfmd_host_id => host_id).where(:sri => filters[:sris]).select(:sidfmd_vul_view_id)
      )
    end
    #CVSS
    if filters[:cvsses].any? then
      label_values = {
        :notreg => nil,
        :d4 => { :top => 10.0 , :bottom => 9.0 },
        :d3 => { :top => 8.9, :bottom => 7.0 },
        :d2 => { :top => 6.9, :bottom => 4.0 },
        :d1 => { :top => 3.9, :bottom => 0.0 },
      }
      conds = []
      filters[:cvsses].each do |label|
        values = label_values[label.to_sym]
        if values then
          conds << "cvss >= #{values[:bottom]} AND cvss <= #{values[:top]}"
        else
          conds << "cvss IS NULL"
        end
      end
      content = content.where(
                              :id => SidfmdVvScsid.where(:sidfmd_host_id => host_id)
                                                  .where("#{conds.join(' OR ')}")
                                                  .select(:sidfmd_vul_view_id)
                            )
    end
    #CVSS指標
    if filters[:cvss_vector].size > 0 then
      if filters[:cvss_vector][:is_target_unregistered].to_s == 'true' then
        content = content.where(
                                :html_id => SidfmdIssue.joins(:sidfmd_issue_cves)
                                                       .where(:sidfmd_issues => { :sidfmd_host_id => host_id })
                                                       .where(:sidfmd_issue_cves => { :c2_vector => nil, :c3_vector => nil })
                                                       .select(:html_id)
                              )
      elsif filters[:cvss_vector][:cvss_v2].values.select{|v| v != nil }.any? or filters[:cvss_vector][:cvss_v3].values.select{|v| v != nil }.any? then
        filter_cvss_v2 = ''
        filter_cvss_v3 = ''
        cvss_v2_vectors = SidfmdIssueCve.get_cvss_v2_vectors.keys.map{|vector| vector.to_sym}
        cvss_v3_vectors = SidfmdIssueCve.get_cvss_v3_vectors.keys.map{|vector| vector.to_sym}
        selected_cvss_v2 = filters[:cvss_vector][:cvss_v2]
        selected_cvss_v3 = filters[:cvss_vector][:cvss_v3]
        is_selected_cvss_v2 = false
        is_selected_cvss_v3 = false
        cvss_v2_vectors.each_with_index do |vector, idx|
          is_selected_cvss_v2 = true if selected_cvss_v2[vector]
          filter_cvss_v2 += '/' unless idx == 0
          filter_cvss_v2 += "#{vector}:#{(selected_cvss_v2[vector] || '.')}"
        end
        cvss_v3_vectors.each_with_index do |vector, idx|
          is_selected_cvss_v3 = true if selected_cvss_v3[vector]
          filter_cvss_v3 += '/' unless idx == 0
          filter_cvss_v3 += "#{vector}:#{(selected_cvss_v3[vector] || '.')}"
        end
        filter_cvss_v2 = "^" + filter_cvss_v2 + "$"
        filter_cvss_v3 = "^" + filter_cvss_v3 + "$"
        #sqlで正規表現を行う
        condition = if is_selected_cvss_v2 and is_selected_cvss_v3 then
                      %| sidfmd_issue_cves.c2_vector ~ '#{filter_cvss_v2}' AND sidfmd_issue_cves.c3_vector ~ '#{filter_cvss_v3}' |
                    elsif is_selected_cvss_v2 then
                      %| sidfmd_issue_cves.c2_vector ~ '#{filter_cvss_v2}' |
                    else
                      %| sidfmd_issue_cves.c3_vector ~ '#{filter_cvss_v3}' |
                    end
        content = content.where(
                                :html_id => SidfmdIssue.joins(:sidfmd_issue_cves)
                                                       .where(:sidfmd_issues => { :sidfmd_host_id => host_id })
                                                       .where(condition)
                                                       .select(:html_id)
                              )
      end
=begin      if filters[:cvss_vector][:is_target_unregistered] then
        condition = %| 
          c2_av IS NULL AND
          c2_ac IS NULL AND
          c2_au IS NULL AND
          c2_c IS NULL AND
          c2_i IS NULL AND
          c2_a IS NULL AND
          c3_av IS NULL AND
          c3_ac IS NULL AND
          c3_pr IS NULL AND
          c3_ui IS NULL AND
          c3_s IS NULL AND
          c3_c IS NULL AND
          c3_i IS NULL AND
          c3_a IS NULL
        |
      else
        cvss_filter_items = []
        filters[:cvss_vector][:cvss_v2].each do |vector, value|
          if value then
            cvss_filter_items << "c2_#{vector.to_s.downcase} = '#{value}'"
          end
        end
        filters[:cvss_vector][:cvss_v3].each do |vector, value|
          if value then
            cvss_filter_items << "c3_#{vector.to_s.downcase} = '#{value}'"
          end
        end
        condition = cvss_filter_items.join(' AND ')
      end
      content = content.where(%|
        sidfmd_vul_views.id IN (
          SELECT sidfmd_vul_view_id
          FROM sidfmd_vv_cves AS cve_table
          WHERE(
            #{condition} AND
            id = (
              SELECT id
              FROM sidfmd_vv_cves AS cve_table2
              WHERE cve_table.sidfmd_vul_view_id = cve_table2.sidfmd_vul_view_id AND cve_table.cve_id = cve_table2.cve_id
              ORDER BY 
              CASE
              WHEN c3_update_datetime IS NOT NULL THEN
                100
              WHEN c2_update_datetime IS NOT NULL THEN
                200
              ELSE
                300
              END,
              c3_update_datetime,
              c2_update_datetime,
              id
              LIMIT 1
            )
          )
        )
      |)
=end
    end

    # SofTek影響範囲 
    if filters[:area] != '' then
      content = content.where('sidfmd_vul_views.html_id > 0').where(:softek_area_id => filters[:area].to_i)
    end
    # 攻撃コード
    if filters[:exploitable].to_s == 'true' then
      content = content.where(
        :id => SidfmdVvScsid.where(:sidfmd_host_id => host_id)
                            .where(:status_id => SidfmdIssue::STATUS_UNFINISHED)
                            .where.not(:cvss => nil)
                            .select(:sidfmd_vul_view_id)
      ).where(:exploitable => true)
    end

    # 対策予定日設定済
    if filters[:due_date] != '::' then
      if /\A(|notset):(|\d{4}\/\d{2}\/\d{2}):(|\d{4}\/\d{2}\/\d{2})\z/ =~ filters[:due_date] then
        notset = ('notset' == Regexp.last_match(1))
        begindate = Regexp.last_match(2)
        enddate = Regexp.last_match(3)
        if notset then
          content = content.where(
            :id => SidfmdVvScsid.where(:sidfmd_host_id => host_id)
                                .where(:due_date => nil)
                                .select(:sidfmd_vul_view_id)
          )
        else
          begin
            if begindate.to_s.size > 0 then
              begindate = Date.parse(begindate)
            else
              begindate = nil
            end
          rescue Exception => e
            log_exception(e)
            begindate = nil
          end

          begin
            if enddate.to_s.size > 0 then
              enddate = Date.parse(enddate)
            else
              enddate = nil
            end
          rescue Exception => e
            log_exception(e)
            enddate = nil
          end

          if begindate and enddate then
            content = content.where(
              :id => SidfmdVvScsid.where(:sidfmd_host_id => host_id)
                                  .where("due_date IS NOT NULL")
                                  .where(:due_date => begindate..enddate)
                                  .select(:sidfmd_vul_view_id)
            )
          elsif begindate then
            content = content.where(
              :id => SidfmdVvScsid.where(:sidfmd_host_id => host_id)
                                  .where("due_date IS NOT NULL")
                                  .where('due_date >= ?', begindate)
                                  .select(:sidfmd_vul_view_id)
            )
          elsif enddate then
            content = content.where(
              :id => SidfmdVvScsid.where(:sidfmd_host_id => host_id)
                                  .where("due_date IS NOT NULL")
                                  .where('due_date <= ?', enddate)
                                  .select(:sidfmd_vul_view_id)
            )
          end
        end
      end
    end

    # キーワード検索
    if filters[:keyword].size > 0 then
      kwd = filters[:keyword].gsub('/', '') #SidfmdVulViewのtitleは"/"で区切られているため、除去して検索
      if kwd.size > 0 then
        if /\A\s*(\d+)\s*\z/ =~ kwd then
          html_id = Regexp.last_match(1)
          content = content.where(%|
            sidfmd_vul_views.html_id = ? OR 
            title ILIKE ? OR 
            sidfmd_vul_views.id IN (SELECT sidfmd_vul_view_id FROM sidfmd_vv_cves WHERE cve_id ILIKE ?)
          |, html_id.to_i, "%#{ActiveRecord::Base.send(:sanitize_sql_like, html_id.to_s.strip)}%", "%#{ActiveRecord::Base.send(:sanitize_sql_like, kwd.strip)}%")
        elsif /\A\s*KB(\d+)\s*\z/i =~ kwd then
          kbid = -Regexp.last_match(1).to_i
          content = content.where(
            'sidfmd_vul_views.html_id = ? OR title ILIKE ?', kbid.to_i, "%#{ActiveRecord::Base.send(:sanitize_sql_like, kwd.strip)}%"
          )
        else
          content = content.where(%|
            sidfmd_vul_views.id IN (SELECT sidfmd_vul_view_id FROM sidfmd_vv_cves WHERE cve_id ILIKE ?) OR 
            title ILIKE ?
          |, "%#{ActiveRecord::Base.send(:sanitize_sql_like, kwd.strip)}%", "%#{ActiveRecord::Base.send(:sanitize_sql_like, kwd.strip)}%")
        end
      else
        content = content.where(:title => '')
      end
    end

    # 日付指定検索
    filters_date_searching_item = filters[:date].select{|k, v| v != '::'}.keys.first
    if filters_date_searching_item then
      if /\A(in|rev):(\d{4}\/\d{2}\/\d{2}|):(\d{4}\/\d{2}\/\d{2}|)\z/ =~ filters[:date][filters_date_searching_item] then
        orig_flg_rev = Regexp.last_match(1)
        orig_begin_date = Regexp.last_match(2)
        orig_end_date = Regexp.last_match(3)
        if orig_flg_rev.size > 0 and orig_begin_date.size == 0 and orig_end_date.size == 0 then
          filters[:date][filters_date_searching_item] = '::'
        else
          flg_rev = orig_flg_rev == 'rev' ? true : false
          begin_date = orig_begin_date.size > 0 ? Date.strptime(orig_begin_date, '%Y/%m/%d') : nil
          end_date = orig_end_date.size > 0 ? Date.strptime(orig_end_date, '%Y/%m/%d') : nil
          if flg_rev then
            if begin_date and end_date then
              begin_date = begin_date.to_time
              end_date = end_date.to_time + DAY_SECONDS - 1
              case filters_date_searching_item
              when :detected_date then
                content = content.where(
                  :id => SidfmdVvScsid.where(:sidfmd_host_id => host_id)
                                      .where.not(:created_on => begin_date..end_date)
                                      .select(:sidfmd_vul_view_id)
                )
              when :created_date then
                content = content.where.not(
                  :html_id => SidfmdHtmlidDate.where('create_datetime BETWEEN ? AND ?', begin_date, end_date).select(:htmlid)
                )
              when :updated_date then
                content = content.where.not(
                  :html_id => SidfmdHtmlidDate.where('update_datetime BETWEEN ? AND ?', begin_date, end_date).select(:htmlid)
                )
              end
            elsif begin_date then
              case filters_date_searching_item
              when :detected_date then
                content = content.where(
                  :id => SidfmdVvScsid.where(:sidfmd_host_id => host_id)
                                      .where.not('sidfmd_vv_scsids.created_on >= ?', begin_date)
                                      .select(:sidfmd_vul_view_id)
                )
              when :created_date then
                content = content.where.not(
                  :html_id => SidfmdHtmlidDate.where('create_datetime >= ?', begin_date).select(:htmlid)
                )
              when :updated_date then
                content = content.where.not(
                  :html_id => SidfmdHtmlidDate.where('update_datetime >= ?', begin_date).select(:htmlid)
                )
              end
            elsif end_date then
              end_date = end_date.to_time + DAY_SECONDS - 1
              case filters_date_searching_item
              when :detected_date then
                content = content.where(
                  :id => SidfmdVvScsid.where(:sidfmd_host_id => host_id)
                                      .where.not('sidfmd_vv_scsids.created_on <= ?', end_date)
                                      .select(:sidfmd_vul_view_id)
                )
              when :created_date then
                content = content.where.not(
                  :html_id => SidfmdHtmlidDate.where('create_datetime <= ?', end_date).select(:htmlid)
                )
              when :updated_date then
                content = content.where.not(
                  :html_id => SidfmdHtmlidDate.where('update_datetime <= ?', end_date).select(:htmlid)
                )
              end
            else
              # last resort
              filters[:date][filters_date_searching_item] = '::'
            end
          else
            if begin_date and end_date then
              begin_date = begin_date.to_time
              end_date = end_date.to_time + DAY_SECONDS - 1
              case filters_date_searching_item
              when :detected_date then
                content = content.where(
                  :id => SidfmdVvScsid.where(:sidfmd_host_id => host_id)
                                      .where(:created_on => begin_date..end_date)
                                      .select(:sidfmd_vul_view_id)
                )
              when :created_date then
                content = content.where(
                  :html_id => SidfmdHtmlidDate.where('create_datetime BETWEEN ? AND ?', begin_date, end_date).select(:htmlid)
                )
              when :updated_date then
                content = content.where(
                  :html_id => SidfmdHtmlidDate.where('update_datetime BETWEEN ? AND ?', begin_date, end_date).select(:htmlid)
                )
              end
            elsif begin_date then
              case filters_date_searching_item
              when :detected_date then
                content = content.where(
                  :id => SidfmdVvScsid.where(:sidfmd_host_id => host_id)
                                      .where('created_on >= ?', begin_date)
                                      .select(:sidfmd_vul_view_id)
                )
              when :created_date then
                content = content.where(
                  :html_id => SidfmdHtmlidDate.where('create_datetime >= ?', begin_date).select(:htmlid)
                )
              when :updated_date then
                content = content.where(
                  :html_id => SidfmdHtmlidDate.where('update_datetime >= ?', begin_date).select(:htmlid)
                )
              end
            elsif end_date then
              end_date = end_date.to_time + DAY_SECONDS - 1
              case filters_date_searching_item
              when :detected_date then
              content = content.where(
                :id => SidfmdVvScsid.where(:sidfmd_host_id => host_id)
                                    .where('created_on <= ?', end_date)
                                    .select(:sidfmd_vul_view_id)
              )
              when :created_date then
                content = content.where(
                  :html_id => SidfmdHtmlidDate.where('create_datetime <= ?', end_date).select(:htmlid)
                )
              when :updated_date then
                content = content.where(
                  :html_id => SidfmdHtmlidDate.where('update_datetime <= ?', end_date).select(:htmlid)
                )
              end
            else
              # last resort
              filters[:date][filters_date_searching_item] = '::'
            end
          end
        end
      else
        filters[:date][filters_date_searching_item] = '::'
      end
    end

    #カスタムタグ検索
    if filters[:ctags] && filters[:ctags][:ids] && filters[:ctags][:ids].any? then
      unless role_admin then
        filters[:ctags][:ids] = filters[:ctags][:ids] & SidfmdCtag.user_relatable_ctag_ids(role_admin, user_category_ids)
      end
      content = content.where(
        :id => SidfmdVvCtag.where(:sidfmd_ctag_id => filters[:ctags][:ids])
                           .where(:sidfmd_host_id => host_id)
                           .select(:sidfmd_vul_view_id)
      )
    end

    content
  end

end
