#!/usr/b/n/env ruby
# -*- coding: utf-8 -*-

module SidfmVulnerabilityControllerSupport
  # 既存のメソッドとメソッド名が被らないように注意

  DAY_SECONDS = 24 * 60 * 60
  # ソート
  # vul_viewはSidfmdVulView
  def sort_vul_content(content:, order:, vul_view:, role_admin:, user_category_ids:, prior_category:, index:, corresp_required:)
    case order
    when 'warning_asc', 'warning_desc' then
      content = content.joins(%|
        LEFT OUTER JOIN sidfmd_vv_scsids ON sidfmd_vv_scsids.sidfmd_host_id = sidfmd_host_views.sidfmd_host_id
      |)
      .where("sidfmd_vv_scsids.sidfmd_vul_view_id = ?", vul_view.id)
      .order(%| 
        CASE 
        WHEN status_id IN (#{SidfmdIssue::STATUS_UNFINISHED.join(', ')}) THEN
          CASE 
          WHEN #{index == :sri ? "sri >= #{corresp_required[:sri]}" : "cvss IS NULL OR cvss >= #{corresp_required[:cvss]}"} THEN
            0
          ELSE
            100
          END
        ELSE
          100
        END
        #{order == 'warning_asc' ? 'ASC' : 'DESC'}
      |)
      .order(%| LOWER(display_name COLLATE "C") #{order == 'warning_asc' ? 'ASC' : 'DESC'} |)
    when 'status_asc', 'status_desc' then
      content = content.joins(%|
        LEFT OUTER JOIN sidfmd_vv_scsids ON sidfmd_vv_scsids.sidfmd_host_id = sidfmd_host_views.sidfmd_host_id
      |)
      .where("sidfmd_vv_scsids.sidfmd_vul_view_id = ?", vul_view.id)
      .order(%| status_id #{order == 'status_asc' ? 'ASC' : 'DESC'}|)
      .order(%| LOWER(display_name COLLATE "C") #{order == 'status_asc' ? 'ASC' : 'DESC'} |)
    when 'sri_asc', 'sri_desc' then
      content = content.joins(%|
        LEFT OUTER JOIN sidfmd_vv_scsids ON sidfmd_vv_scsids.sidfmd_host_id = sidfmd_host_views.sidfmd_host_id
      |)
      .where("sidfmd_vv_scsids.sidfmd_vul_view_id = ?", vul_view.id)
      .order(%| sri #{order == 'sri_asc' ? 'DESC' : 'ASC'}|)
      .order(%| LOWER(display_name COLLATE "C") #{order == 'sri_asc' ? 'ASC' : 'DESC'} |)
    when 'cvss_asc', 'cvss_desc' then
      content = content.joins(%|
        LEFT OUTER JOIN sidfmd_vv_scsids ON sidfmd_vv_scsids.sidfmd_host_id = sidfmd_host_views.sidfmd_host_id
      |)
      .where("sidfmd_vv_scsids.sidfmd_vul_view_id = ?", vul_view.id)
      .order(%| CASE WHEN cvss IS NOT NULL THEN cvss ELSE 0.0 END #{order == 'cvss_asc' ? 'DESC' : 'ASC'}|)
      .order(%| LOWER(display_name COLLATE "C") #{order == 'cvss_asc' ? 'ASC' : 'DESC'} |)
    when 'exploitable_asc', 'exploitable_desc' then
      if vul_view.html_id > 0 and vul_view.exploitable then
        content = content.joins(%|
          LEFT OUTER JOIN sidfmd_vv_scsids ON sidfmd_vv_scsids.sidfmd_host_id = sidfmd_host_views.sidfmd_host_id
        |)
        .where("sidfmd_vv_scsids.sidfmd_vul_view_id = ?", vul_view.id)
        .order(%|
          CASE WHEN cvss IS NOT NULL THEN
            CASE
            WHEN status_id IN (#{SidfmdIssue::STATUS_UNFINISHED.join(', ')}) THEN
              500
            ELSE
              100
            END
          ELSE
            25
          END
          #{order == 'exploitable_asc' ? 'DESC' : 'ASC'}
        |)
        .order(%| LOWER(display_name COLLATE "C") #{order == 'exploitable_asc' ? 'ASC' : 'DESC'} |)
      else
        content = content.order(%| LOWER(display_name COLLATE "C") #{order == 'exploitable_asc' ? 'ASC' : 'DESC'} |)
      end
    when 'elapseddays_asc', 'elapseddays_desc' then
      content = content.joins(%|
        LEFT OUTER JOIN sidfmd_vv_scsids ON sidfmd_vv_scsids.sidfmd_host_id = sidfmd_host_views.sidfmd_host_id
      |)
      .where("sidfmd_vv_scsids.sidfmd_vul_view_id = ?", vul_view.id)
      .order(%|
        CASE
        WHEN status_id IN (SELECT unnest(ARRAY[#{SidfmdIssue::STATUS_UNFINISHED.join(', ')}])) THEN
          sidfmd_vv_scsids.created_on
        ELSE
          NULL
        END
        #{order == 'elapseddays_asc' ? 'ASC NULLS LAST' : 'DESC NULLS FIRST'}
      |)
      .order(%| LOWER(display_name COLLATE "C") #{order == 'elapseddays_asc' ? 'ASC' : 'DESC'} |)
    when 'hostname_asc', 'hostname_desc' then
      content = content.order(%| LOWER(display_name COLLATE "C") #{order == 'hostname_asc' ? 'ASC' : 'DESC'} |)
    when 'categories_asc', 'categories_desc' then
      if prior_category == 'custom' then
        content = content.joins(%|
          LEFT OUTER JOIN(
            SELECT 
              sidfmd_host_view_id AS order_custom_category_host_view_id,
              MIN(LOWER(name COLLATE "C")) AS order_custom_category_name
            FROM
              sidfmd_hv_categories
            #{role_admin ? '' : "WHERE sidfmd_category_id IN (SELECT unnest(ARRAY[#{user_category_ids.join(', ')}]))"}
            GROUP BY sidfmd_host_view_id
          ) AS order_custom_min_name_category
          ON sidfmd_host_views.id = order_custom_min_name_category.order_custom_category_host_view_id
        |)
        .order(%|
          CASE WHEN order_custom_category_name IS NULL THEN
            system_category_id
          ELSE
            0
          END
          #{order == 'categories_asc' ? 'ASC' : 'DESC'}
        |)
        .order(%|
          LOWER(order_custom_category_name COLLATE "C")
          #{order == 'categories_asc' ? 'ASC' : 'DESC'}
        |)
        .order(%| LOWER(display_name COLLATE "C") #{order == 'categories_asc' ? 'ASC' : 'DESC'} |)
      else
        content = content.order(:system_category_id => order == 'categories_asc' ? :asc : :desc)
                                                   .order(%| LOWER(display_name COLLATE "C") #{order == 'categories_asc' ? 'ASC' : 'DESC'} |)
      end
    when 'os_asc', 'os_desc' then
      content = content.order(%|
        CASE WHEN osname = '' OR osname = '---' THEN
          500
        ELSE
          0
        END
      |)
      .order(%| LOWER(osname COLLATE "C") #{order == 'os_asc' ? 'ASC' : 'DESC'} |)
      .order(%| LOWER(display_name COLLATE "C") #{order == 'os_asc' ? 'ASC' : 'DESC'} |)
    when 'journals_asc', 'journals_desc' then
      content = content.joins(%|
        LEFT OUTER JOIN(
          SELECT 
            sidfmd_host_id,
            COUNT(journals.id) AS count_journals
          FROM
            sidfmd_issues INNER JOIN issues ON sidfmd_issues.issue_id = issues.id
            LEFT OUTER JOIN journals ON journals.journalized_id = issues.id
          WHERE sidfmd_issues.html_id = #{vul_view.html_id}
          GROUP BY sidfmd_host_id
        ) AS order_count_journals
        ON sidfmd_host_views.sidfmd_host_id = order_count_journals.sidfmd_host_id
      |)
      .order(%| count_journals #{order == 'journals_asc' ? 'DESC' : 'ASC'} |)
      .order(%| LOWER(display_name COLLATE "C") #{order == 'journals_asc' ? 'ASC' : 'DESC'} |)
    when 'duedate_asc', 'duedate_desc' then
      content = content.joins(%|
        LEFT OUTER JOIN sidfmd_vv_scsids ON sidfmd_vv_scsids.sidfmd_host_id = sidfmd_host_views.sidfmd_host_id
      |)
      .where("sidfmd_vv_scsids.sidfmd_vul_view_id = ?", vul_view.id)
      .order(%| DATE_TRUNC('DAY', due_date) #{order == 'duedate_asc' ? 'DESC NULLS LAST' : 'ASC NULLS FIRST'} |)
      .order(%| LOWER(display_name COLLATE "C") #{order == 'duedate_asc' ? 'ASC' : 'DESC'} |)
    end
    content

  end

  #ホストのOS一覧
  def vul_dist_versions(content)
    dist_versions =
      if content.class != Array then
        content.where('LENGTH(osname) >= 1').order(:osname).distinct.pluck(:osname)
      else
        content.collect {|x|
          (x[:os] and  x[:os][:osname] and x[:os][:osname].size > 0) ? x[:os][:osname] : nil
        }.compact.uniq.sort
      end
  end

  # フィルタの初期値
  def default_vul_filters(dist_versions)
    filters = {}
    filters[:warning] = { :values => [ 'warning', 'others', 'all' ].map(&:freeze).freeze, :default => 'all'.freeze }
    filters[:status] = {
      :values => (
        SidfmdIssue::STATUS_ALL.collect {|x| x.to_s } +
                    [ SidfmdIssue::STATUS_UNFINISHED.collect {|x| x.to_s }.join(':') ] +
                    [ SidfmdIssue::STATUS_FINISHED.collect {|x| x.to_s }.join(':') ] +
                    [ '0' ]
      ).map(&:freeze).freeze, :default => '0'.freeze
    }
    filters[:sris] = {
      :values => SidfmdIssueSri::VALUES.map{|v| [v.to_s, v]}.to_h,
      :default => [].freeze
    }
    filters[:cvsses] = {
      :values => SidfmdIssueCve::DISPLAY_SORTED_LABELS.map{|v| [v.to_s, v]}.to_h,
      :default => [].freeze
    }
    filters[:exploitable] = { :values => [ 'true', '' ].map(&:freeze).freeze, :default => ''.freeze }
    filters[:categories] = {
      :type_source => { #type_sourceはdefaultでは型が不明な場合に使う。各値には期待されるデータと同じ型のデータを入れる
        :ids => [0].freeze,
        :is_and => false,
        :is_except => false
      },
      :values => nil,
      :default => {
        :ids => [].freeze,
        :is_and => false,
        :is_except => false
      }
    }
    filters[:os] = { :values => dist_versions + [ ''.freeze ], :default => '' }
    filters[:due_date] = { :values => nil, :default => '::' }
    filters[:date] = { :values => nil, :default => '::' }
    filters[:keyword] = { :values => nil, :default => '' }
    filters[:journal] = { :values => nil, :default => { :period => '::', :filter_status_change => false, :filter_duedate_change => false } }
    filters[:products] = {
      :type_source => {
        :product_all_version_ids => [0],
        :product_ids => [0],
        :version_ids => [0],
        :lproduct_all_version_ids => [0],
        :lproduct_ids => [0],
        :lversion_ids => [0]
      },
      :values => nil,
      :default => {
        :product_all_version_ids => [],
        :product_ids => [],
        :version_ids => [],
        :lproduct_all_version_ids => [],
        :lproduct_ids => [],
        :lversion_ids => []
      }
    }
    filters[:ctags] = {
      :values => nil,
      :default => { :ids => [] },
      :type_source => {
        :ids => [0]
      }
    }
    filters
  end

  # フィルタ
  # vul_viewはSidfmdVulView
  def filter_vul_content(content:, filters:, vul_view:, role_admin:, user_category_ids:, index:, corresp_required:)
    #プロダクト検索
    if filters[:products].values.select{|v| v.size > 0 }.any? then
      prods_related_host_ids = []
      prod_info = filters[:products].symbolize_keys
      if prod_info[:product_all_version_ids] and prod_info[:product_all_version_ids].any? then
        #全バージョン
        prods_related_host_ids |= SidfmdHostApp.where(:product_id => prod_info[:product_all_version_ids]).pluck(:sidfmd_host_id)
      end

      if prod_info[:product_ids] and prod_info[:product_ids].any? then
        #「全て」のバージョン
        prods_related_host_ids |= SidfmdHostApp.where(:product_id => prod_info[:product_ids])
                                               .where('version_id = 0 OR version_id IS NULL')#バージョンは該当させない
                                               .pluck(:sidfmd_host_id)
      end

      if prod_info[:version_ids] and prod_info[:version_ids].any? then
        #versionが検索対象
        prods_related_host_ids |= SidfmdHostApp.where(:version_id => prod_info[:version_ids]).pluck(:sidfmd_host_id)
      end

      if prod_info[:lproduct_all_version_ids] and prod_info[:lproduct_all_version_ids].any? then
        #全バージョン
        prods_related_host_ids |= SidfmdHostLapp.where(:sidfmd_lproduct_id => prod_info[:lproduct_all_version_ids]).pluck(:sidfmd_host_id)
      end

      if prod_info[:lproduct_ids] and prod_info[:lproduct_ids].any? then
        #「全て」のバージョン
        prods_related_host_ids |= SidfmdHostLapp.where(:sidfmd_lproduct_id => prod_info[:lproduct_ids])
                                                .where(:sidfmd_lversion_id => 0)#バージョンは該当させない
                                                .pluck(:sidfmd_host_id)
      end

      if prod_info[:lversion_ids] and prod_info[:lversion_ids].any? then
        #lversionが検索対象
        prods_related_host_ids |= SidfmdHostLapp.where(:sidfmd_lversion_id => prod_info[:lversion_ids]).pluck(:sidfmd_host_id)
      end

      if prods_related_host_ids.any? then
        content = content.where(:sidfmd_host_id => prods_related_host_ids)
      else
        content = content.where(:sidfmd_host_id => 0)
      end
    end

    # 要注意
    if filters[:warning] == 'warning' then
      content = content.where(
        :sidfmd_host_id => SidfmdVvScsid.where(%|
          sidfmd_vul_view_id = #{vul_view.id} AND
          status_id IN (SELECT unnest(ARRAY[#{SidfmdIssue::STATUS_UNFINISHED.join(', ')}])) AND
          #{index == :sri ? "sri >= #{corresp_required[:sri]}" : "(cvss IS NULL OR cvss >= #{corresp_required[:cvss]})"}
        |).select(:sidfmd_host_id)
      )
    elsif filters[:warning] == 'others' then
      content = content.where(
        :sidfmd_host_id => SidfmdVvScsid.where(%|
          sidfmd_vul_view_id = #{vul_view.id} AND
          (status_id NOT IN (SELECT unnest(ARRAY[#{SidfmdIssue::STATUS_UNFINISHED.join(', ')}])) OR
          #{index == :sri ? "sri < #{corresp_required[:sri]}" : "(cvss IS NOT NULL AND cvss < #{corresp_required[:cvss]})"})
        |).select(:sidfmd_host_id)
      )
    end

    # 対応状況
    if /\A(\d):(\d)\z/ =~ filters[:status] then
      status1 = Regexp.last_match(1).to_i
      status2 = Regexp.last_match(2).to_i
      content = content.where(
        :sidfmd_host_id => SidfmdVvScsid.where(:sidfmd_vul_view_id => vul_view.id)
                                        .where(:status_id => [ status1, status2 ])
                                        .select(:sidfmd_host_id)
      )
    elsif filters[:status] != '0' then
      content = content.where(
        :sidfmd_host_id => SidfmdVvScsid.where(:sidfmd_vul_view_id => vul_view.id)
                                        .where(:status_id => filters[:status].to_i)
                                        .select(:sidfmd_host_id)
      )
    end

    # SRI
    if filters[:sris].any? then
      content = content.where(
        :sidfmd_host_id => SidfmdVvScsid.where(:sidfmd_vul_view_id => vul_view.id)
                                        .where(:sri => filters[:sris])
                                        .select(:sidfmd_host_id)
      )
    end
    # CVSS
    if filters[:cvsses].any? then
      label_values = {
        :notreg => nil,
        :d4 => { :top => 10.0 , :bottom => 9.0 },
        :d3 => { :top => 8.9, :bottom => 7.0 },
        :d2 => { :top => 6.9, :bottom => 4.0 },
        :d1 => { :top => 3.9, :bottom => 0.0 },
      }
      conds = []
      filters[:cvsses].each do |label|
        values = label_values[label.to_sym]
        if values then
          conds << "cvss >= #{values[:bottom]} AND cvss <= #{values[:top]}"
        else
          conds << "cvss IS NULL"
        end
      end
      content = content.where(
                               :sidfmd_host_id => SidfmdVvScsid.where(:sidfmd_vul_view_id => vul_view.id)
                                                               .where("#{conds.join(' OR ')}")
                                                               .select(:sidfmd_host_id)
                             )
    end

    # 攻撃コード
    if filters[:exploitable] == 'true' then
      if vul_view.exploitable then
        content = content.where(
          :sidfmd_host_id => SidfmdVvScsid.where(:sidfmd_vul_view_id => vul_view.id)
                                          .where(:status_id => SidfmdIssue::STATUS_UNFINISHED)
                                          .select(:sidfmd_host_id)
        )
      else
        content = content.where(:id => 0)
      end
    end

    #カテゴリ(複数)
    if filters[:categories][:ids].any? then
      unless role_admin then
        filters[:categories][:ids] = (filters[:categories][:ids].map(&:to_i) & user_category_ids) + SidfmdCategory.where(:is_system => true).where(:id => filters[:categories][:ids]).pluck(:id)
      end
      target_host_ids = []
      if filters[:categories][:is_and].to_s == 'true' then
        SidfmdHost.where(
          :id => SidfmdCategoriesHost.where(:sidfmd_category_id => filters[:categories][:ids]).select(:sidfmd_host_id)
        ).where(:flg_delete => false)
        .each do |host|
          host_cat_ids = host.sidfmd_categories.pluck(:id)
          if (filters[:categories][:ids] - host_cat_ids).empty? then
            target_host_ids << host.id
          end
        end
      else
        target_host_ids = SidfmdCategoriesHost.where(:sidfmd_category_id => filters[:categories][:ids])
                                              .select(:sidfmd_host_id)
      end
      content = if filters[:categories][:is_except].to_s == 'true' then
                               content.where.not(:sidfmd_host_id => target_host_ids)
                             else
                               content.where(:sidfmd_host_id => target_host_ids)
                             end
    end

    # OS
    if filters[:os] != '' then
      content = content.where(:osname => filters[:os])
    end

    # 履歴
    if filters[:journal][:period] != '::' then
      if /^(in|rev):(\d{4}\/\d{2}\/\d{2}|):(\d{4}\/\d{2}\/\d{2}|)$/ =~ filters[:journal][:period] then
        orig_flg_rev = Regexp.last_match(1)
        orig_begin_date = Regexp.last_match(2)
        orig_end_date = Regexp.last_match(3)
        if orig_flg_rev.size > 0 and orig_begin_date.size == 0 and orig_end_date.size == 0 then
          filters[:journal][:period] = '::'
        else
          flg_rev = orig_flg_rev == 'rev'
          begin_date = orig_begin_date.size > 0 ? Date.strptime(orig_begin_date, '%Y/%m/%d') : nil
          end_date = orig_end_date.size > 0 ? Date.strptime(orig_end_date, '%Y/%m/%d') : nil
          begin_date = begin_date.to_time if begin_date
          end_date = end_date.to_time + DAY_SECONDS - 1 if end_date

          j_host_ids = Journal.joins(:issue)
                              .joins('INNER JOIN sidfmd_issues ON sidfmd_issues.issue_id = issues.id')
                              .joins('LEFT OUTER JOIN journal_details ON journal_details.journal_id = journals.id')
                              .where('sidfmd_issues.html_id = ?', vul_view.html_id)
                              .where('sidfmd_issues.sidfmd_host_id IN (?)', content.select(:sidfmd_host_id))
                              .select('sidfmd_issues.sidfmd_host_id')
          if flg_rev then
            if begin_date and end_date then
              j_host_ids = j_host_ids.where.not(:journals => { :created_on => begin_date..end_date })
            elsif begin_date then
              j_host_ids = j_host_ids.where.not('journals.created_on >= ?', begin_date)
            elsif end_date then
              j_host_ids = j_host_ids.where.not('journals.created_on <= ?', end_date)
            else
              # last resort
              filters[:journal][:period] = '::'
            end
          else
            if begin_date and end_date then
              j_host_ids = j_host_ids.where(:journals => { :created_on => begin_date..end_date })
            elsif begin_date then
              j_host_ids = j_host_ids.where('journals.created_on >= ?', begin_date)
            elsif end_date then
              j_host_ids = j_host_ids.where('journals.created_on <= ?', end_date)
            else
              # last resort
              filters[:journal][:period] = '::'
            end
          end
          if filters[:journal][:period] != '::' then
            if filters[:journal][:filter_status_change].to_s == 'true' and filters[:journal][:filter_duedate_change].to_s == 'true' then
              f_host_ids = j_host_ids.where(:journal_details => { :prop_key => 'status_id' }).pluck(:sidfmd_host_id) &
                           j_host_ids.where(:journal_details => { :prop_key => 'due_date' }).pluck(:sidfmd_host_id)
              if f_host_ids.any? then
                j_host_ids = j_host_ids.where(%|
                  sidfmd_issues.sidfmd_host_id IN (SELECT unnest(ARRAY[#{f_host_ids.join(', ')}]))
                |)
              else
                j_host_ids = j_host_ids.where(:journals => { :id => 0 })
              end
            elsif filters[:journal][:filter_status_change].to_s == 'true' then
              j_host_ids = j_host_ids.where(:journal_details => { :prop_key => 'status_id' })
            elsif filters[:journal][:filter_duedate_change].to_s == 'true' then
              j_host_ids = j_host_ids.where(:journal_details => { :prop_key => 'due_date' })
            end
            content = content.where(:sidfmd_host_id => j_host_ids)
          end
        end
      else
        filters[:journal][:period] = '::'
      end
    end

    # 対策予定日設定済
    if filters[:due_date] != '::' then
      if /\A(|notset):(|\d{4}\/\d{2}\/\d{2}):(|\d{4}\/\d{2}\/\d{2})\z/ =~ filters[:due_date] then
        notset = ('notset' == Regexp.last_match(1))
        begindate = Regexp.last_match(2)
        enddate = Regexp.last_match(3)
        if notset then
          content = content.where(
            :sidfmd_host_id => SidfmdVvScsid.where(:due_date => nil).where(:sidfmd_vul_view_id => vul_view.id).select(:sidfmd_host_id)
          )
        else
          begin
            if begindate.to_s.size > 0 then
              begindate = Date.parse(begindate)
            else
              begindate = nil
            end
          rescue Exception => e
            log_exception(e)
            begindate = nil
          end
          begin
            if enddate.to_s.size > 0 then
              enddate = Date.parse(enddate)
            else
              enddate = nil
            end
          rescue Exception => e
            log_exception(e)
            enddate = nil
          end
          if begindate and enddate then
            content = content.where(
              :sidfmd_host_id => SidfmdVvScsid.where("due_date IS NOT NULL")
                                              .where(:due_date => begindate..enddate)
                                              .where(:sidfmd_vul_view_id => vul_view.id)
                                              .select(:sidfmd_host_id)
            )
          elsif begindate then
            content = content.where(
              :sidfmd_host_id => SidfmdVvScsid.where("due_date IS NOT NULL")
                                              .where('due_date >= ?', begindate)
                                              .where(:sidfmd_vul_view_id => vul_view.id)
                                              .select(:sidfmd_host_id)
            )
          elsif enddate then
            content = content.where(
              :sidfmd_host_id => SidfmdVvScsid.where("due_date IS NOT NULL")
                                              .where('due_date <= ?', enddate)
                                              .where(:sidfmd_vul_view_id => vul_view.id)
                                              .select(:sidfmd_host_id)
            )
          end
        end
      end
    end

    # キーワード検索
    if filters[:keyword].size > 0 then
      kwd = filters[:keyword].strip
      content = content.where('hostname ILIKE ? OR display_name ILIKE ?', "%#{ActiveRecord::Base.send(:sanitize_sql_like, kwd)}%", "%#{ActiveRecord::Base.send(:sanitize_sql_like, kwd)}%")
    end

    # 日付指定検索
    if filters[:date] != '::' then
      if /^(in|rev):(\d{4}\/\d{2}\/\d{2}|):(\d{4}\/\d{2}\/\d{2}|)$/ =~ filters[:date] then
        orig_flg_rev = Regexp.last_match(1)
        orig_begin_date = Regexp.last_match(2)
        orig_end_date = Regexp.last_match(3)
        if orig_flg_rev.size > 0 and orig_begin_date.size == 0 and orig_end_date.size == 0 then
          filters[:date] = '::'
        else
          flg_rev = orig_flg_rev == 'rev' ? true : false
          begin_date = orig_begin_date.size > 0 ? Date.strptime(orig_begin_date, '%Y/%m/%d') : nil
          end_date = orig_end_date.size > 0 ? Date.strptime(orig_end_date, '%Y/%m/%d') : nil
          if flg_rev then
            if begin_date and end_date then
              begin_date = begin_date.to_time
              end_date = end_date.to_time + DAY_SECONDS - 1
              content = content.where(
                :sidfmd_host_id => SidfmdVvScsid.where(:sidfmd_vul_view_id => vul_view.id)
                                                .where.not(:created_on => begin_date..end_date)
                                                .select(:sidfmd_host_id)
              )
            elsif begin_date then
              begin_date = begin_date.to_time
              content = content.where(
                :sidfmd_host_id => SidfmdVvScsid.where(:sidfmd_vul_view_id => vul_view.id)
                                                .where.not('created_on >= ?', begin_date)
                                                .select(:sidfmd_host_id)
              )
            elsif end_date then
              end_date = end_date.to_time + DAY_SECONDS - 1
              content = content.where(
                :sidfmd_host_id => SidfmdVvScsid.where(:sidfmd_vul_view_id => vul_view.id)
                                                .where.not('created_on <= ?', end_date)
                                                .select(:sidfmd_host_id)
              )
            else
              # last resort
              filters[:date] = '::'
            end
          else
            if begin_date and end_date then
              begin_date = begin_date.to_time
              end_date = end_date.to_time + DAY_SECONDS - 1
              content = content.where(
                :sidfmd_host_id => SidfmdVvScsid.where(:sidfmd_vul_view_id => vul_view.id)
                                                .where(:created_on => begin_date..end_date)
                                                .select(:sidfmd_host_id)
              )
            elsif begin_date then
              begin_date = begin_date.to_time
              content = content.where(
                :sidfmd_host_id => SidfmdVvScsid.where(:sidfmd_vul_view_id => vul_view.id)
                                                .where('created_on >= ?', begin_date)
                                                .select(:sidfmd_host_id)
              )
            elsif end_date then
              end_date = end_date.to_time + DAY_SECONDS - 1
              content = content.where(
                :sidfmd_host_id => SidfmdVvScsid.where(:sidfmd_vul_view_id => vul_view.id)
                                                .where('created_on <= ?', end_date)
                                                .select(:sidfmd_host_id)
              )
            else
              # last resort
              filters[:date] = '::'
            end
          end
        end
      else
        filters[:date] = '::'
      end
    end

    #カスタムタグ検索
    if filters[:ctags] && filters[:ctags][:ids] && filters[:ctags][:ids].any? then
      unless role_admin then
        filters[:ctags][:ids] = filters[:ctags][:ids] & SidfmdCtag.user_relatable_ctag_ids(role_admin, user_category_ids)
      end
      content = content.where(
        :id => SidfmdHvCtag.where(:sidfmd_ctag_id => filters[:ctags][:ids])
                           .where(:html_id => vul_view.html_id)
                           .select(:sidfmd_host_view_id)
      )
    end

    content

  end

end
